Package: SpotClean
Version: 1.14.0
Date: 2024/06/08
Title: SpotClean adjusts for spot swapping in spatial transcriptomics
        data
Authors@R: c(
    person("Zijian", "Ni", email="zni25@wisc.edu", role = c("aut", "cre"), 
        comment = c(ORCID = "0000-0003-1181-8337")),
    person("Christina", "Kendziorski", role = "ctb")
  )
Depends: R (>= 4.2.0),
Imports: stats, methods, utils, dplyr, S4Vectors, SummarizedExperiment,
        SpatialExperiment, Matrix, rhdf5, ggplot2, grid, readbitmap,
        rjson, tibble, viridis, grDevices, RColorBrewer, Seurat, rlang
Suggests: testthat (>= 2.1.0), knitr, BiocStyle, rmarkdown, R.utils,
        spelling
biocViews: DataImport, RNASeq, Sequencing, GeneExpression, Spatial,
        SingleCell, Transcriptomics, Preprocessing
Description: 
    SpotClean is a computational method to adjust for spot swapping in spatial 
    transcriptomics data. Recent spatial transcriptomics experiments utilize 
    slides containing thousands of spots with spot-specific barcodes that bind 
    mRNA. Ideally, unique molecular identifiers at a spot measure spot-specific 
    expression, but this is often not the case due to bleed from nearby spots, 
    an artifact we refer to as spot swapping. SpotClean is able to estimate the 
    contamination rate in observed data and decontaminate the spot swapping 
    effect, thus increase the sensitivity and precision of downstream analyses. 
License: GPL-3
NeedsCompilation: yes
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://github.com/zijianni/SpotClean
BugReports: https://github.com/zijianni/SpotClean/issues
Language: en-US
git_url: https://git.bioconductor.org/packages/SpotClean
git_branch: RELEASE_3_23
git_last_commit: fbf937f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-29 01:01:29 UTC; biocbuild
Author: Zijian Ni [aut, cre] (ORCID: <https://orcid.org/0000-0003-1181-8337>),
  Christina Kendziorski [ctb]
Maintainer: Zijian Ni <zni25@wisc.edu>

# Generated by roxygen2: do not edit by hand

S3method(arcScore,SummarizedExperiment)
S3method(arcScore,default)
S3method(spotclean,SpatialExperiment)
S3method(spotclean,SummarizedExperiment)
S3method(visualizeHeatmap,SummarizedExperiment)
S3method(visualizeHeatmap,default)
S3method(visualizeLabel,SummarizedExperiment)
S3method(visualizeLabel,default)
export(arcScore)
export(convertToSeurat)
export(createSlide)
export(keepHighGene)
export(read10xRaw)
export(read10xRawH5)
export(read10xSlide)
export(spotclean)
export(visualizeHeatmap)
export(visualizeLabel)
export(visualizeSlide)
import(Matrix)
import(grid)
importClassesFrom(Matrix,dgCMatrix)
importFrom(Matrix,colSums)
importFrom(Matrix,readMM)
importFrom(Matrix,rowMeans)
importFrom(Matrix,sparseMatrix)
importFrom(RColorBrewer,brewer.pal)
importFrom(S4Vectors,metadata)
importFrom(Seurat,"DefaultAssay<-")
importFrom(Seurat,Cells)
importFrom(Seurat,CreateSeuratObject)
importFrom(Seurat,FindVariableFeatures)
importFrom(Seurat,GetTissueCoordinates)
importFrom(Seurat,NormalizeData)
importFrom(Seurat,Read10X_Image)
importFrom(SpatialExperiment,scaleFactors)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(dplyr,filter)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(ggplot2,Geom)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,ggproto_parent)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_set)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(grDevices,colorRampPalette)
importFrom(methods,as)
importFrom(readbitmap,read.bitmap)
importFrom(rhdf5,h5ls)
importFrom(rhdf5,h5read)
importFrom(rjson,fromJSON)
importFrom(rlang,.data)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,lm)
importFrom(stats,optim)
importFrom(stats,quantile)
importFrom(tibble,tibble)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importFrom(viridis,scale_fill_viridis)
importMethodsFrom(S4Vectors,"metadata<-")
importMethodsFrom(S4Vectors,metadata)

# SpotClean: a computational method to adjust for spot swapping in spatial transcriptomics data

SpotClean is a computational method to adjust for spot swapping in spatial transcriptomics data. Recent spatial transcriptomics experiments utilize slides containing thousands of spots with spot-specific barcodes that bind mRNA. Ideally, unique molecular identifiers at a spot measure spot-specific expression, but this is often not the case due to bleed from nearby spots, an artifact we refer to as spot swapping. SpotClean is able to estimate the contamination rate in observed data and decontaminate the spot swapping effect, thus increase the sensitivity and precision of downstream analyses.

### Introduction

Spatial transcriptomics (ST), named [Method of the Year 2020](https://www.nature.com/articles/s41592-020-01033-y) by *Nature Methods* in 2020, is a powerful and widely-used experimental method for profiling genome-wide gene expression across a tissue. In a typical ST experiment, fresh-frozen (or FFPE) tissue is sectioned and placed onto a slide containing spots, with each spot containing millions of capture oligonucleotides with spatial barcodes unique to that spot. The tissue is imaged, typically via Hematoxylin and Eosin (H&E) staining. Following imaging, the tissue is permeabilized to release mRNA which then binds to the capture oligonucleotides, generating a cDNA library consisting of transcripts bound by barcodes that preserve spatial information. Data from an ST experiment consists of the tissue image coupled with RNA-sequencing data collected from each spot. A first step in processing ST data is tissue detection, where spots on the slide containing tissue are distinguished from background spots without tissue. Unique molecular identifier (UMI) counts at each spot containing tissue are then used in downstream analyses.

Ideally, a gene-specific UMI at a given spot would represent expression of that gene at that spot, and spots without tissue would show no (or few) UMIs. This is not the case in practice. Messenger RNA bleed from nearby spots causes substantial contamination of UMI counts, an artifact we refer to as spot swapping. On average, we observe that more than 30% of UMIs at a tissue spot did not originate from this spot, but from other spots contaminating it. Spot swapping confounds downstream inferences including normalization, marker gene-based annotation, differential expression and cell type decomposition.

We developed **SpotClean** to adjust for the effects of spot swapping in ST experiments. SpotClean is able to measure the per-spot contamination rates in observed data and decontaminate gene expression levels, thus increases the sensitivity and precision of downstream analyses. Our package `SpotClean` is built based on 10x Visium spatial transcriptomics experiments, currently the most widely-used commercial protocol, providing functions to load raw spatial transcriptomics data from 10x Space Ranger outputs, decontaminate the spot swapping effect, estimate contamination levels, visualize expression profiles and spot labels on the slide, and connect with other widely-used packages for further analyses. SpotClean can be potentially extended to other spatial transcriptomics data as long as the gene expression data in both tissue and background regions are available.


### Installation

Install the GitHub version:

```{r}
if(!requireNamespace("devtools", quietly = TRUE))
    install.packages("devtools")

devtools::install_github("zijianni/SpotClean", build_manual = TRUE, build_vignettes = TRUE)

```

Install the Bioconductor version:

```{r}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("SpotClean")

```

Load package after installation:

```{r}
library(SpotClean)
```

### Tutorial

After installing the package, access the vignette by running

```{r}
vignette("SpotClean")
```

### Citation

We appreciate it if you could cite our work when using `SpotClean`:

Ni, Z., Prasad, A., Chen, S. et al. SpotClean adjusts for spot swapping in spatial transcriptomics data. *Nat Commun* 13, 2971 (2022). https://doi.org/10.1038/s41467-022-30587-y

A BibTeX entry for LaTeX users can be found by running 

```{r}
citation("SpotClean")
```

## ----echo=FALSE, results="hide", message=FALSE--------------------------------
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
set.seed(1)

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("SpotClean")
# 

## -----------------------------------------------------------------------------
library(SpotClean)
library(S4Vectors)

## ----eval=FALSE---------------------------------------------------------------
# # Not run
# 
# # Load 10x Visium data
# mbrain_raw <- read10xRaw("/path/to/matrix/folder")
# mbrain_slide_info <- read10xSlide("/path/to/tissue/csv",
#                                   "/path/to/tissue/image",
#                                   "/path/to/scale/factor")
# 
# # Visualize raw data
# mbrain_obj <- createSlide(count_mat = mbrain_raw,
#                           slide_info = mbrain_slide_info)
# visualizeSlide(slide_obj = mbrain_obj)
# visualizeHeatmap(mbrain_obj,rownames(mbrain_raw)[1])
# 
# # Decontaminate raw data
# decont_obj <- spotclean(mbrain_obj)
# 
# # Visualize decontaminated gene
# visualizeHeatmap(decont_obj,rownames(mbrain_raw)[1])
# 
# # Visualize the estimated per-spot contamination rate
# visualizeHeatmap(decont_obj,metadata(decont_obj)$contamination_rate,
#                  logged = FALSE, legend_title = "contamination rate",
#                  legend_range = c(0,1))
# 
# # (Optionally) Transform to Seurat object for downstream analyses
# seurat_obj <- convertToSeurat(decont_obj,image_dir = "/path/to/spatial/folder")

## ----eval=F-------------------------------------------------------------------
# # Not run
# library(SpatialExperiment)
# slide_obj <- read10xVisium(samples = "/path/to/spaceranger/output/",
#                          data = "raw") # must specify data = "raw"
# decont_obj <- spotclean(slide_obj)
# str(assays(decont_obj)$decont)

## -----------------------------------------------------------------------------
# load count matrix
data(mbrain_raw)
str(mbrain_raw)

# read spatial metadata
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
list.files(spatial_dir)

mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))
str(mbrain_slide_info)

## -----------------------------------------------------------------------------
slide_obj <- createSlide(mbrain_raw, mbrain_slide_info)
slide_obj

## ----fig.width=5, fig.height=5------------------------------------------------
visualizeSlide(slide_obj)

## ----fig.width=5, fig.height=4------------------------------------------------
visualizeLabel(slide_obj,"tissue")

## ----fig.width=5, fig.height=4------------------------------------------------
metadata(slide_obj)$slide$total_counts <- Matrix::colSums(mbrain_raw)
visualizeHeatmap(slide_obj,"total_counts")

## ----fig.width=5, fig.height=4------------------------------------------------
visualizeHeatmap(slide_obj,"Mbp")

## -----------------------------------------------------------------------------
decont_obj <- spotclean(slide_obj, maxit=10, candidate_radius = 20)

## -----------------------------------------------------------------------------
decont_obj
names(metadata(decont_obj))

## ----fig.width=5, fig.height=4------------------------------------------------
visualizeHeatmap(decont_obj,"Mbp")

## -----------------------------------------------------------------------------
summary(metadata(decont_obj)$contamination_rate)

## -----------------------------------------------------------------------------
arcScore(slide_obj)

## -----------------------------------------------------------------------------
seurat_obj <- convertToSeurat(decont_obj,image_dir = spatial_dir)

## -----------------------------------------------------------------------------
sessionInfo()

#' @name read10xRaw
#'
#' @title Read 10x Space Ranger output data
#'
#' @description \code{read10xRaw()} is a one-line handy function for reading
#' the raw expression data from 10x Space Ranger outputs and producing a count
#' matrix as an R object.
#'
#' \code{read10xRawH5()} is for reading 10x Space Ranger
#' output HDF5 file (ended with .h5).
#'
#' \code{read10xSlide()} is for reading slide
#' information (e.g. spot positions) and the tissue image from 10x
#' Space Ranger outputs. This function is developed based on 10x's secondary
#' analysis pipeline
#' https://support.10xgenomics.com/spatial-gene-expression/software/pipelines/latest/rkit.
#'
#' @param count_dir (chr) The directory of 10x output matrix data.
#' The directory should include
#' three files: barcodes.tsv.gz, features.tsv.gz, matrix.mtx.gz.
#'
#' @param h5_file (chr) The path of 10x output matrix HDF5 file
#' (ended with .h5).
#'
#' @param tissue_csv_file (chr) The path of 10x output CSV file of
#' spot positions, usually named \code{tissue_positions_list.csv} for 
#' Space Ranger V1 and \code{tissue_positions.csv} for Space Ranger V2.
#'
#' @param tissue_img_file (chr) The path of the 10x output low resolution
#' tissue image in PNG format,
#' usually named \code{tissue_lowres_image.png}. If \code{NULL},
#' the returned slide data does not contain image
#' information. Please do provide this file if you could find it.
#' Default: \code{NULL}
#'
#' @param scale_factor_file (chr) The path of the 10x output scale factor
#' file in json format, usually named \code{scalefactors_json.json}.
#' If \code{NULL}, spot positions in image will
#' not be corrected by the scale factor. Please do provide this file
#' if you could find it. Default: \code{NULL}
#'
#' @param row_name (chr) Specify either using gene symbols
#' (\code{row_name = "symbol"}) or gene Ensembl IDs (\code{row_name = "id"})
#' as row names of the count matrix.
#' Default: \code{row_name = "symbol"}
#'
#' @param meta (logical) If \code{TRUE}, \code{read10xRaw} or
#' \code{read10xRawH5} returns a list containing both the
#' count matrix and metadata of genes (features). Metadata includes feature
#' names, IDs and other additional information depending on Space Ranger
#' output. If \code{FALSE}, only returns the count matrix.
#' Default: \code{FALSE}
#'
#' @return If \code{meta = TRUE}, \code{read10xRaw()} or \code{read10xRawH5()}
#' returns a list of two elements: a
#' "dgCMatrix" sparse matrix containing expression counts and a data
#' frame containing metadata of genes (features). For the count matrix,
#' each row is a gene (feature) and each column is a spot barcode. If
#' \code{meta = FALSE}, only returns the count matrix.
#'
#' \code{read10xSlide()} returns a list of two objects. The first object, slide,
#' is a data.frame where each row corresponds to a spot
#' and each column corresponds to slide information such as row and column
#' positions on the slide. The second object, grob, is a Grid Graphical Object
#' of the tissue image when specifying \code{tissue_img_file}.
#'
#' @examples
#'
#' # simulate 10x output files of count matrix
#' data(mbrain_raw)
#' data_dir <- file.path(tempdir(),"sim_example")
#' dir.create(data_dir)
#' matrix_dir <- file.path(data_dir,"matrix.mtx")
#' barcode_dir <- gzfile(file.path(data_dir, "barcodes.tsv.gz"), open="wb")
#' gene_dir <- gzfile(file.path(data_dir, "features.tsv.gz"), open="wb")
#'
#' # For simplicity, use gene names to generate gene IDs to fit the format.
#' gene_name <- rownames(mbrain_raw)
#' gene_id <- paste0("ENSG_fake_",gene_name)
#' barcode_id <- colnames(mbrain_raw)
#'
#' Matrix::writeMM(mbrain_raw,file = matrix_dir)
#' write(barcode_id,file = barcode_dir)
#' write.table(cbind(gene_id,gene_name,"type"),file = gene_dir,
#'     sep = "\t", quote = FALSE, col.names = FALSE, row.names = FALSE)
#' R.utils::gzip(matrix_dir)
#' close(barcode_dir)
#' close(gene_dir)
#'
#'
#' # read expression count matrix
#' list.files(data_dir)
#' mbrain_raw_new <- read10xRaw(data_dir)
#' str(mbrain_raw_new)
#' identical(mbrain_raw, mbrain_raw_new)
#'
#' # read slide metadata
#' spatial_dir <- system.file(file.path("extdata",
#'                                      "V1_Adult_Mouse_Brain_spatial"),
#'                            package = "SpotClean")
#' list.files(spatial_dir)
#' mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
#'                                        "tissue_positions_list.csv"),
#'              tissue_img_file = file.path(spatial_dir,
#'                                        "tissue_lowres_image.png"),
#'              scale_factor_file = file.path(spatial_dir,
#'                                        "scalefactors_json.json"))
#' str(mbrain_slide_info)


#' @rdname Read10x
#' @importFrom utils read.delim
#' @importFrom Matrix readMM
#' @importFrom Matrix sparseMatrix
#'
#'
#' @export

read10xRaw <- function(count_dir = NULL,
                       row_name = "symbol",
                       meta = FALSE) {
    if(is.null(count_dir)){
        count_dir <- getwd()
    }
    if (!row_name %in% c("symbol", "id"))
        stop("row_name should be either \"symbol\" or \"id\".")
    if (!dir.exists(count_dir))
        stop("Directory does not exist.")
    count_dir <- gsub("/$", "", count_dir)
    fname <- list.files(count_dir)
    
    # Path to files
    
    if(!all(c("barcodes.tsv.gz","features.tsv.gz",
              "matrix.mtx.gz")%in%fname)){
        stop("No 10x output file detected.")
    }
    Barcode <- file.path(count_dir, "barcodes.tsv.gz")
    Gene <- file.path(count_dir, "features.tsv.gz")
    CountMat <- file.path(count_dir, "matrix.mtx.gz")
    
    # Read gene and barcode info
    
    barcode <- readLines(Barcode)
    gene.meta <- read.delim(Gene, header = FALSE, colClasses = "character")
    colnames(gene.meta) <- c("id", "symbol", "type")
    
    if (row_name == "symbol") {
        #gene names as row names of count matrix
        gene <- gene.meta$symbol
    } else{
        #gene ids as row names of count matrix
        gene <- gene.meta$id
    }
    
    # Read count matrix
    
    countmat <- as(readMM(CountMat), "CsparseMatrix")
    colnames(countmat) <- barcode
    rownames(countmat) <- make.unique(gene)
    
    if (meta) {
        #return a list including count matrix and gene metadata
        return(list(CountMatrix = countmat, Metadata = gene.meta))
    } else{
        #only return count matrix
        return(countmat)
    }
}



#' @rdname Read10x
#' @importFrom rhdf5 h5ls
#' @importFrom rhdf5 h5read
#'
#' @export

read10xRawH5 <- function(h5_file,
                         row_name = "symbol",
                         meta = FALSE) {
    
    if (!row_name %in% c("symbol", "id"))
        stop("row_name should be either \"symbol\" or \"id\".")
    if (!file.exists(h5_file))
        stop("File does not exist.")
    
    fname <- h5ls(h5_file)
    
    data.temp <-  h5read(h5_file, "/matrix")
    barcode <- data.temp$barcodes
    gene.meta <- data.temp$features
    
    if (row_name == "symbol") {
        #gene names as row names of count matrix
        gene <- gene.meta$name
    } else{
        #gene ids as row names of count matrix
        gene <- gene.meta$id
    }
    
    countmat <-
        sparseMatrix(
            i = data.temp$indices,
            p = data.temp$indptr,
            x = as.numeric(data.temp$data),
            index1 = FALSE,
            dims = data.temp$shape
        )
    colnames(countmat) <- as.vector(barcode)
    rownames(countmat) <- make.unique(gene)
    
    if (meta) {
        #return a list including count matrix and gene metadata
        return(list(CountMatrix = countmat, Metadata = gene.meta))
    } else{
        #only return count matrix
        return(countmat)
    }
    
}

#' @rdname Read10x
#' @importFrom utils read.csv
#' @importFrom readbitmap read.bitmap
#' @import grid
#' @importFrom rjson fromJSON
#' @importFrom methods as
#'
#' @export

read10xSlide <- function(tissue_csv_file,
                         tissue_img_file = NULL,
                         scale_factor_file = NULL){
    
    # Load tissue information
    slide <- read.csv(tissue_csv_file,
                      col.names=c("barcode","tissue","row",
                                  "col","imagerow","imagecol"),
                      header = FALSE)
    
    # For Space Ranger V2, re-read CSV with header=TRUE
    if(inherits(slide$tissue,"character")){
        slide <- read.csv(tissue_csv_file,
                          col.names=c("barcode","tissue","row",
                                      "col","imagerow","imagecol"),
                          header = TRUE)
    }
    
    slide$tissue <- as.factor(slide$tissue)
    
    grob <- NULL
    # Load tissue image
    if(!is.null(tissue_img_file)){
        
        # Load downsampled image
        tissue_img_file <- read.bitmap(tissue_img_file)
        
        grob <- rasterGrob(tissue_img_file,
                           width=unit(1,"npc"),
                           height=unit(1,"npc"))
        
        slide$height <- nrow(tissue_img_file)
        slide$width <- ncol(tissue_img_file)
    }
    
    # Load scale factor
    if(!is.null(scale_factor_file)){
        scales <- rjson::fromJSON(file = scale_factor_file)
        slide$imagerow <- slide$imagerow * scales$tissue_lowres_scalef
        slide$imagecol <- slide$imagecol * scales$tissue_lowres_scalef
    }
    
    return(list(slide=slide, grob=grob))
}

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(
    tissue_csv_file=file.path(spatial_dir,
                              "tissue_positions_list.csv"),
    tissue_img_file = file.path(spatial_dir,
                                "tissue_lowres_image.png"),
    scale_factor_file = file.path(spatial_dir,
                                  "scalefactors_json.json"))

mbrain_obj <- createSlide(mbrain_raw, mbrain_slide_info)

background_bcs <- dplyr::filter(mbrain_slide_info$slide,tissue==0)$barcode

test_that("Invalid background indices",{
    expect_error(arcScore(mbrain_raw, "foo"),
                 "Invalid background barcodes")
})

test_that("Calculation",{
    expect_equal(arcScore(mbrain_raw, background_bcs),
                 0.05160659)
    expect_equal(arcScore(mbrain_raw, background_bcs[1:100]),
                 0.00414822)
    expect_equal(arcScore(mbrain_obj),
                 0.05160659)
})

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(
    tissue_csv_file=file.path(spatial_dir,
                              "tissue_positions_list.csv"),
    tissue_img_file = file.path(spatial_dir,
                                "tissue_lowres_image.png"),
    scale_factor_file = file.path(spatial_dir,
                                  "scalefactors_json.json"))

mbrain_obj <- createSlide(mbrain_raw, mbrain_slide_info)

mbrain_slide_info2 <- mbrain_slide_info
mbrain_slide_info2$slide <- head(mbrain_slide_info2$slide, 100)
mbrain_obj2 <- createSlide(mbrain_raw[,mbrain_slide_info2$slide$barcode],
                           mbrain_slide_info2)

test_that("Non-existing column",{
    expect_error(visualizeLabel(mbrain_obj, label="foo"),
                 "Label name does not exist")
    expect_error(visualizeLabel(mbrain_slide_info$slide, label="foo"),
                 "Label name does not exist")

})

test_that("Invalid input label vector",{
    expect_error(visualizeLabel(mbrain_obj, label=rnorm(10)),
                 "Invalid label input")
    expect_error(visualizeLabel(mbrain_slide_info$slide, label=rnorm(10)),
                 "Invalid label input")

})

gp1 <- visualizeLabel(mbrain_obj, subset_barcodes = colnames(mbrain_obj2))
gp2 <- visualizeLabel(mbrain_obj2)

test_that("Subsetting barcodes", {
    expect_identical(gp1$data, gp2$data[match(gp1$data$barcode,
                                              gp2$data$barcode),])

})

test_that("Object class", {
    expect_s3_class(gp1,c("gg","ggplot"))

})

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(
    tissue_csv_file=file.path(spatial_dir,
                              "tissue_positions_list.csv"),
    tissue_img_file = file.path(spatial_dir,
                                "tissue_lowres_image.png"),
    scale_factor_file = file.path(spatial_dir,
                                  "scalefactors_json.json"))

# Simulate 10x output files

data_dir <- file.path(tempdir(),"sim_example")
if(dir.exists(data_dir)){
    file.remove(list.files(data_dir,full.names = TRUE))
}else{
    dir.create(data_dir)
}

matrix_dir <- file.path(data_dir,"matrix.mtx")
barcode_dir <- gzfile(file.path(data_dir, "barcodes.tsv.gz"), open="wb")
gene_dir <- gzfile(file.path(data_dir, "features.tsv.gz"), open="wb")

# For simplicity, use gene names to generate gene IDs to fit the format.
gene_name <- rownames(mbrain_raw)
gene_id <- paste0("ENSG_fake_",gene_name)
barcode_id <- colnames(mbrain_raw)

Matrix::writeMM(mbrain_raw,file = matrix_dir)
write(barcode_id,file = barcode_dir)
write.table(cbind(gene_id,gene_name,"type"),file = gene_dir,
    sep = "\t", quote = FALSE, col.names = FALSE, row.names = FALSE)
R.utils::gzip(matrix_dir)
close(barcode_dir)
close(gene_dir)

# test reading function

test_that("Directory/files existence", {
    expect_error(read10xRaw(paste0(data_dir,"/foo")),
                 "Directory does not exist")
    expect_error(read10xRaw(tempdir()), "No 10x output file detected")
})

test_that("Data loading", {
    expect_identical(read10xRaw(data_dir), mbrain_raw)
})


test_that("Metadata loading", {
    expect_true(is.list(read10xRaw(data_dir, meta = TRUE)))
})

# test reading slide metadata
test_that("Metadata loading", {
    expect_identical(mbrain_slide_info$slide[1,1], "ACGCCTGACACGCGCT-1")
    expect_equal(mbrain_slide_info$slide[1,5], 63.84281593)
})

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(
    tissue_csv_file=file.path(spatial_dir,
                              "tissue_positions_list.csv"),
    tissue_img_file = file.path(spatial_dir,
                                "tissue_lowres_image.png"),
    scale_factor_file = file.path(spatial_dir,
                                  "scalefactors_json.json"))

mbrain_obj <- createSlide(mbrain_raw, mbrain_slide_info)

gp <- visualizeSlide(mbrain_obj)

test_that("Object class", {
    expect_s3_class(gp,c("gg","ggplot"))

})

S4Vectors::metadata(mbrain_obj)$grob <- NULL

test_that("Missing image grob", {
    expect_error(visualizeSlide(mbrain_obj),
                 "No valid image information")

})

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(
    tissue_csv_file=file.path(spatial_dir,
                              "tissue_positions_list.csv"),
    tissue_img_file = file.path(spatial_dir,
                                "tissue_lowres_image.png"),
    scale_factor_file = file.path(spatial_dir,
                                  "scalefactors_json.json"))


test_that("Barcodes not matching", {
    expect_warning(createSlide(mbrain_raw[,1:10], mbrain_slide_info),
                 "Barcodes in count matrix do not match")
})

test_that("Rearrange barcode order", {
    slide <- mbrain_slide_info$slide
    mbrain_slide_info_2 <- mbrain_slide_info
    mbrain_slide_info_2$slide <- slide[sample(nrow(slide)),]

    expect_identical(createSlide(mbrain_raw, mbrain_slide_info),
                     createSlide(mbrain_raw, mbrain_slide_info_2))
})

slide_obj <- createSlide(mbrain_raw, mbrain_slide_info, gene_cutoff = 100)

test_that("Object class", {
    expect_s4_class(slide_obj,"SummarizedExperiment")
})

test_that("Gene filtering", {
    expect_true(nrow(slide_obj)==14)
})

test_that("input data frame",{
    expect_warning(createSlide(mbrain_raw, mbrain_slide_info$slide),
                   "Input slide info does not contain image")
})

data(mbrain_raw)

spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))

mbrain_obj <- createSlide(mbrain_raw, mbrain_slide_info)

test_that("Invalid image directory",{
    expect_error(convertToSeurat(mbrain_obj, "foo", "raw"),
                 "tissue_lowres_image.png")
})

seurat_obj <- convertToSeurat(mbrain_obj,image_dir = spatial_dir,
                              filter_matrix = FALSE)

test_that("Consistent count matrix",{
    if(as.integer(gsub("\\<(\\d+)\\.\\d+\\.\\d+", "\\1", 
                       seurat_obj@version))>=5){
        seurat_counts <- seurat_obj@assays$Spatial$counts
    }else{
        seurat_counts <- seurat_obj@assays$Spatial@counts
    }
    
    expect_identical(mbrain_obj@assays@data$raw,
                     seurat_counts)
})

test_that("Consistent slide metadata",{
    # Seurat VisiumV2 has dropped the `coordinates` slot so need to remove 
    # some tests here
    # expect_identical(factor(seurat_obj@images$slice1@coordinates$tissue),
    # mbrain_obj@metadata$slide$tissue)
    expect_identical(seurat_obj@images$slice1$centroids@cells,
                     mbrain_obj@metadata$slide$barcode)
    # expect_identical(seurat_obj@images$slice1@coordinates[,c("row","col")],
    #                  mbrain_obj@metadata$slide[,c("row","col")])
})

seurat_obj_f <- convertToSeurat(mbrain_obj,image_dir = spatial_dir,
                              filter_matrix = TRUE)

test_that("Filter background spots",{
    expect_equal(dim(seurat_obj_f), c(100, 2702))
    expect_identical(
        sort(colnames(seurat_obj_f)),
        sort(mbrain_slide_info$slide$barcode[mbrain_slide_info$slide$tissue==1])
        )
})

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(
    tissue_csv_file=file.path(spatial_dir,
                              "tissue_positions_list.csv"),
    tissue_img_file = file.path(spatial_dir,
                                "tissue_lowres_image.png"),
    scale_factor_file = file.path(spatial_dir,
                                  "scalefactors_json.json"))

mbrain_obj <- createSlide(mbrain_raw, mbrain_slide_info)

mbrain_slide_info2 <- mbrain_slide_info
mbrain_slide_info2$slide <- head(mbrain_slide_info2$slide, 100)
mbrain_obj2 <- createSlide(mbrain_raw[,mbrain_slide_info2$slide$barcode],
                           mbrain_slide_info2)


test_that("Non-existing gene",{
    expect_error(visualizeHeatmap(mbrain_obj, value="foo"),
                 "Specified gene does not exist")
    expect_error(visualizeHeatmap(mbrain_slide_info$slide,
                                  value="foo",mbrain_raw),
                 "Specified gene does not exist")

})

test_that("Missing input matrix",{
    expect_error(visualizeHeatmap(mbrain_slide_info$slide, value="foo"),
                 "You must provide an input expression")

})

test_that("Invalid input value vector",{
    expect_error(visualizeHeatmap(mbrain_obj, value=rnorm(10)),
                 "Invalid value input")
    expect_error(visualizeHeatmap(mbrain_obj, value=rnorm(10), mbrain_raw),
                 "Invalid value input")

})

test_that("Matrix not match slide barcodes",{
    gp1 <- visualizeHeatmap(mbrain_slide_info$slide, value="Bc1",
                     mbrain_raw[,mbrain_slide_info2$slide$barcode])
    gp2 <- visualizeHeatmap(mbrain_slide_info2$slide, value="Bc1",
                            mbrain_raw)
    expect_identical(gp1$data, gp2$data)

})

gp1 <- visualizeHeatmap(mbrain_obj, value="Bc1",
                      subset_barcodes = colnames(mbrain_obj2))
gp2 <- visualizeHeatmap(mbrain_obj2, value="Bc1")

gp3 <- visualizeHeatmap(mbrain_slide_info$slide, value="Bc1", mbrain_raw,
                        subset_barcodes = colnames(mbrain_obj2))
gp4 <- visualizeHeatmap(mbrain_slide_info2$slide, value="Bc1", mbrain_raw)

test_that("Subsetting barcodes", {
    expect_identical(gp1$data, gp2$data[match(gp1$data$barcode,
                                              gp2$data$barcode),])
    expect_identical(gp3$data, gp4$data[match(gp3$data$barcode,
                                              gp4$data$barcode),])
})

test_that("Object class", {
    expect_s3_class(gp1,c("gg","ggplot"))
    expect_s3_class(gp3,c("gg","ggplot"))
})

#' @title Filter and return highly expressed or highly variable genes
#'
#' @description This function is for filtering genes in the expression count
#' matrix based on their average expression and variability. Usually genes
#' with low expressions and variability are less interesting and do not
#' contribute too much to downstream analyses, but rather bring technical noise.
#' It is always recommended to pre-filter gene expression matrix before
#' any analyses.
#'
#' We apply the
#' function \code{FindVariableFeatures} with \code{selection.method = "mvp"}
#' from package \code{Seurat} on log transformed expression matrix to
#' detect high variable genes. This method does not require a pre-specified
#' number of high variable genes.
#'
#' @param count_mat (matrix of num) Input count matrix to be filtered.
#' Can be either standard matrix format or sparse matrix format.
#'
#' @param top_high (int) Only look for highly expressed and variable genes
#' within this number of top expressed genes. Default: 5000.
#'
#' @param mean_cutoff (num) Genes with average expressions among all
#' spots exceeding this cutoff are kept as highly expressed genes.
#'
#' @param return_matrix (logical) Whether return filtered matrix instead
#' of gene names. Default: \code{FALSE}
#'
#' @param verbose (logical) Whether print progress information.
#' Default: \code{TRUE}
#'
#' @return A vector of gene names or a filtered expression count
#' matrix with the same class as \code{count_mat}.
#'
#' @examples
#'
#' data(mbrain_raw)
#' dim(mbrain_raw)
#'
#' mbrain_raw_f <- keepHighGene(mbrain_raw, mean_cutoff=100,
#'                              return_matrix=TRUE)
#' dim(mbrain_raw_f)
#'
#' @importFrom Seurat CreateSeuratObject NormalizeData FindVariableFeatures
#' @importFrom Matrix rowMeans
#'
#'
#' @export


keepHighGene <- function(count_mat, top_high=5000,
                         mean_cutoff=1, return_matrix=FALSE,
                         verbose=TRUE){

    mean_exp <- rowMeans(count_mat)
    # keep at most this number of highly expressed genes.
    top_genes <- rank(-mean_exp)<=top_high
    count_mat <- count_mat[top_genes,,drop=FALSE]
    mean_exp <- mean_exp[top_genes]

    high_exp_genes <- mean_exp>=mean_cutoff

    S_vf <- NormalizeData(CreateSeuratObject(count_mat), verbose = FALSE)
    S_vf <- FindVariableFeatures(S_vf,
                                 selection.method = "mvp", verbose = FALSE)
    
    # accommodate changes in Seurat v5 objects
    if(as.integer(gsub("\\<(\\d+)\\.\\d+\\.\\d+", "\\1", S_vf@version))>=5){
        high_variable_genes <- S_vf@assays$RNA@meta.data$vf_mvp_data_variable
    }else{
        high_variable_genes <- S_vf@assays$RNA@meta.features$mvp.variable
    }
    
    gene_tokeep <- high_variable_genes | high_exp_genes

    if(verbose){
        message("Kept ",sum(gene_tokeep),
                " highly expressed or highly variable genes.")
    }

    if(return_matrix){
        return(count_mat[gene_tokeep,, drop=FALSE])
    }else{
        return(names(which(gene_tokeep)))
    }

}

data(mbrain_raw)

mbrain_filter <- keepHighGene(mbrain_raw, return_matrix=TRUE)

test_that("Default parameters", {
    expect_true(nrow(mbrain_filter)==100)

})

test_that("Object class", {
    expect_s4_class(mbrain_filter, "dgCMatrix")
})

test_that("Top high gene cutoff", {
    expect_true(length(keepHighGene(mbrain_raw, top_high = 50, mean_cutoff = 1))==50)

})

test_that("Mean expression cutoff", {
    expect_true(length(keepHighGene(mbrain_raw, mean_cutoff = Inf))==11)
    expect_true(length(keepHighGene(mbrain_raw, mean_cutoff = 500))==14)
})

#' @title Visualize spot labels on the 2D slide
#'
#' @description Generate and visualize spot labels on the 2D slide
#' as a ggplot2 object. Spot labels can be given via parameter \code{label} as
#' external input or one column in the \code{slide} slot of the slide
#' object's metadata via \code{label_col}. Exactly one of \code{label}
#' and \code{label_col} must be specified.
#'
#' @param object A slide object created or
#' inherited from \code{createSlide()}, or a \code{data.frame} of slide
#' information with columns: barcodes, tissue, imagerow, imagecol, etc.
#'
#' @param ... Arguments passed to other methods
#'
#' @return A \code{ggplot2} object.
#'
#' @examples
#'
#' data(mbrain_raw)
#' spatial_dir <- system.file(file.path("extdata",
#'                                      "V1_Adult_Mouse_Brain_spatial"),
#'                            package = "SpotClean")
#' mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
#'                                        "tissue_positions_list.csv"),
#'              tissue_img_file = file.path(spatial_dir,
#'                                        "tissue_lowres_image.png"),
#'              scale_factor_file = file.path(spatial_dir,
#'                                        "scalefactors_json.json"))
#' mbrain_obj <- createSlide(mbrain_raw,
#'                           mbrain_slide_info)
#' gp <- visualizeLabel(mbrain_obj, label="tissue",
#'                      title="mbrain", legend_title="tissue or background")
#' plot(gp)
#'
#' @rdname visualizeLabel
#'
#' @export

visualizeLabel <- function(object, ...) {
    UseMethod(generic = "visualizeLabel", object = object)
}


#' @param label (categorical vector or chr) Either a vector of labels for all
#' spots, or the column name in the \code{slide} slot of the slide
#' object's metadata. In the former case, the order of values in the vector
#' should match the spot barcodes in the slide object. In the latter case,
#' The column should be categorical with reasonably number of unique labels.
#' Default: \code{"tissue"}
#'
#' @param subset_barcodes (vector of chr) A subset of spot barcodes to plot.
#' By default it plots all spots in the slide object. This can be useful
#' when only plotting tissue spots or specific tissue types or regions.
#' Default: \code{NULL}
#'
#' @param title (chr) Title of the plot. Default: \code{""}
#'
#' @param legend_title (chr) Title of the legend. Under default,
#' use \code{label} as legend title. Default: \code{NULL}
#'


#' @importFrom methods as
#'
#' @method visualizeLabel default
#' @rdname visualizeLabel
#'
#' @export

visualizeLabel.default <- function(object, label="tissue",
                                   subset_barcodes=NULL, title="",
                                   legend_title=NULL, ...){

    if (!inherits(x = object, "data.frame")) {
        object <- as(object = object, Class = "data.frame")
    }

    slide <- object

    gp <- .visualize_label(slide, label,
                           subset_barcodes, title,
                           legend_title, ...)
    return(gp)

}

#' @method visualizeLabel SummarizedExperiment
#' @rdname visualizeLabel
#' @importFrom S4Vectors metadata
#' @export
#'
visualizeLabel.SummarizedExperiment <- function(object, label="tissue",
                                                subset_barcodes=NULL, title="",
                                                legend_title=NULL, ...){

    slide <- metadata(object)$slide

    gp <- .visualize_label(slide, label,
                           subset_barcodes, title,
                           legend_title, ...)
    return(gp)

}

#' @importFrom ggplot2 ggplot aes geom_point coord_cartesian xlim ylim
#' @importFrom ggplot2 xlab ylab ggtitle labs guides guide_legend
#' @importFrom ggplot2 theme_set theme_bw theme element_blank element_line
#' @importFrom dplyr filter
#' @importFrom rlang .data
#'
.visualize_label <- function(slide, label="tissue",
                             subset_barcodes=NULL, title="",
                             legend_title=NULL, ...){

    # manipulate label to plot
    if(length(label)==1){

        if(!label%in%colnames(slide)){
            stop("Label name does not exist in slide metadata.")
        }
        if(is.null(legend_title)){
            legend_title <- label
        }
        label <- slide[,label]

    }else if(length(label)==nrow(slide)){

        label <- as.character(label)

    }else{
        stop("Invalid label input.")
    }

    slide$label <- label

    # subsetting barcodes
    if(!is.null(subset_barcodes)){
        slide <- filter(slide, .data$barcode%in%subset_barcodes)
    }

    # plot
    gp <- ggplot(slide, aes(x=.data$imagecol,y=.data$imagerow,
                            fill=factor(.data$label))) +
        geom_point(shape = 21, size = 1.75, color="white")+
        coord_cartesian(expand=FALSE)+
        xlim(0,max(slide$width))+
        ylim(max(slide$height),0)+
        xlab("") +
        ylab("") +
        ggtitle(title)+
        labs(fill = legend_title)+
        guides(fill = guide_legend(override.aes = list(size=3)))+
        theme_set(theme_bw(base_size = 10))+
        theme(panel.grid.major = element_blank(),
              panel.grid.minor = element_blank(),
              panel.background = element_blank(),
              axis.line = element_line(colour = "black"),
              axis.text = element_blank(),
              axis.ticks = element_blank())

    return(gp)


}

#' @title Visualize the Visium slide image
#'
#' @description Generate and visualize the tissue image as a ggplot2 object.
#' Users can manually add and modify layers (e.g. title, axis)
#' following ggplot2's syntax.
#'
#' @param slide_obj A slide object created or inherited from
#' \code{createSlide()}.
#'
#' @param title (chr) Title of the plot. Default: \code{""}
#'
#' @return A \code{ggplot2} object.
#'
#' @examples
#'
#' data(mbrain_raw)
#' spatial_dir <- system.file(file.path("extdata",
#'                                      "V1_Adult_Mouse_Brain_spatial"),
#'                            package = "SpotClean")
#' mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
#'                                        "tissue_positions_list.csv"),
#'              tissue_img_file = file.path(spatial_dir,
#'                                        "tissue_lowres_image.png"),
#'              scale_factor_file = file.path(spatial_dir,
#'                                        "scalefactors_json.json"))
#' mbrain_obj <- createSlide(mbrain_raw,
#'                           mbrain_slide_info)
#' gp <- visualizeSlide(mbrain_obj)
#' plot(gp)


#' @importFrom ggplot2 ggplot aes coord_cartesian xlim ylim xlab ylab
#' @importFrom ggplot2 ggtitle guides guide_legend theme_set theme_bw
#' @importFrom ggplot2 theme element_blank element_line
#' @importFrom tibble tibble
#' @importFrom S4Vectors metadata
#' @importFrom rlang .data
#'
#' @export

visualizeSlide <- function(slide_obj, title=""){

    slide <- metadata(slide_obj)$slide
    grob <- metadata(slide_obj)$grob

    if(is.null(grob)){
        stop("No valid image information. ",
        "Check path to image file in read10xSlide().")
    }

    gp <- ggplot(slide, aes(x = .data$imagecol, y = .data$imagerow)) +
        .geom_spatial(data = tibble(grob=list(grob)),
                      aes(grob = grob),
                      x = 0.5,
                      y = 0.5) +
        coord_cartesian(expand = FALSE) +
        xlim(0, max(slide$width)) +
        ylim(max(slide$height), 0) +
        xlab("") +
        ylab("") +
        ggtitle(title)+
        guides(fill = guide_legend(override.aes = list(size = 3))) +
        theme_set(theme_bw(base_size = 10)) +
        theme(
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.background = element_blank(),
            axis.line = element_line(colour = "black"),
            axis.text = element_blank(),
            axis.ticks = element_blank()
        )

    return(gp)
}


#' @import grid
#' @importFrom ggplot2 ggproto Geom ggproto_parent layer
# This function is developed based on 10x's secondary analysis pipeline
# https://support.10xgenomics.com/spatial-gene-expression/software/pipelines/latest/rkit.

.geom_spatial <-  function(mapping = NULL,
                          data = NULL,
                          stat = "identity",
                          position = "identity",
                          na.rm = FALSE,
                          show.legend = NA,
                          inherit.aes = FALSE,
                          ...) {

    GeomCustom <- ggproto(
        "GeomCustom",
        Geom,
        setup_data = function(self, data, params) {
            data <- ggproto_parent(Geom, self)$setup_data(data, params)
            data
        },

        draw_group = function(data, panel_scales, coord) {
            vp <- viewport(x=data$x, y=data$y)
            g <- editGrob(data$grob[[1]], vp=vp)
            .ggname(".geom_spatial", g)
        },

        required_aes = c("grob","x","y")

    )

    layer(
        geom = GeomCustom,
        mapping = mapping,
        data = data,
        stat = stat,
        position = position,
        show.legend = show.legend,
        inherit.aes = inherit.aes,
        params = list(na.rm = na.rm, ...)
    )
}


# Name ggplot grid object, from ggplot2/R/utilities-grid.r
# Convenience function to name grid objects

.ggname <- function(prefix, grob) {
    grob$name <- grobName(grob, prefix)
    grob
}

#' @title Example 10x Visium spatial data: raw count matrix
#'
#' @description  This dataset contains one sparse count matrix
#' \code{mbrain_raw} of gene expressions. The original dataset can be found at
#' https://support.10xgenomics.com/spatial-gene-expression/datasets/1.0.0/V1_Adult_Mouse_Brain.
#' For simplicity, we only keep the top 100 highest expressed genes in this
#' example data.
#'
#' If users read raw data using \code{read10xRaw} (or \code{read10xRawH5}),
#' they should get exactly the same format as the object in this dataset.
#'
#' @docType data
#'
#' @usage data(mbrain_raw)
#'
#' @format An object of class \code{"dgCMatrix"}.
#'
#' @keywords datasets
#'
#' @source \href{https://support.10xgenomics.com/spatial-gene-expression/datasets/1.0.0/V1_Adult_Mouse_Brain}{V1_Adult_Mouse_Brain}
#'
#' @examples
#' data(mbrain_raw)
#' str(mbrain_raw)
#'
#' @importClassesFrom Matrix dgCMatrix
"mbrain_raw"

#' @title Convert slide object to Seurat object
#'
#' @description This function converts our slide object of class
#' \code{SummarizedExperiment} to Seurat object of class \code{Seurat} so that
#' users can directly proceed with Seurat spatial analyses pipelines.
#' Built based on Seurat's \code{Load10X_Spatial()}.
#'
#' @param slide_obj A slide object created or inherited from
#' \code{createSlide()}.
#'
#' @param image_dir (chr) Path to directory with 10X Genomics visium image data;
#' should include files \code{tissue_lowres_image.png},
#' \code{scalefactors_json.json} and \code{tissue_positions_list.csv}.
#'
#' @param filter_matrix (logical) If \code{TRUE}, only keep spots that have been
#' determined to be over tissue. If \code{slide_obj} only contains tissue spots,
#' \code{filter_matrix} has to be set \code{TRUE}. If \code{slide_obj}
#' contains both tissue and background spots, setting \code{filter_matrix=TRUE}
#' will subset the expression matrix to tissue spots only. Default: \code{TRUE}
#'
#' @param slice (chr) Name for the stored image of the tissue slice.
#' Default: "slice1"
#'
#' @return A Seurat object with spatial information.
#'
#' @examples
#'
#' # load count matrix and slide metadata
#' data(mbrain_raw)
#' spatial_dir <- system.file(file.path("extdata",
#'                                      "V1_Adult_Mouse_Brain_spatial"),
#'                            package = "SpotClean")
#' mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
#'                                        "tissue_positions_list.csv"),
#'              tissue_img_file = file.path(spatial_dir,
#'                                        "tissue_lowres_image.png"),
#'              scale_factor_file = file.path(spatial_dir,
#'                                        "scalefactors_json.json"))
#'
#' # Create slide object
#' mbrain_obj <- createSlide(mbrain_raw,
#'                           mbrain_slide_info)
#'
#' # Convert to Seurat object
#' seurat_obj <- convertToSeurat(mbrain_obj, spatial_dir, "raw")
#' str(seurat_obj)
#'
#' @importFrom Seurat Read10X_Image Cells CreateSeuratObject DefaultAssay<-
#' @importFrom Seurat GetTissueCoordinates
#' @importFrom SummarizedExperiment assay
#' @importFrom S4Vectors metadata
#' @export

convertToSeurat <- function(slide_obj, image_dir,
                            slice="slice1",
                            filter_matrix=TRUE){

    # create Seurat object
    object <- CreateSeuratObject(assay(slide_obj),
                                 assay = "Spatial")

    # load image and add to Seurat object
    image <- Read10X_Image(image.dir = image_dir,
                           filter.matrix = filter_matrix)
    ts_coord <- GetTissueCoordinates(image)

    if(nrow(ts_coord)>ncol(object)){
        stop("The slide object has fewer spots than the image data.
Consider setting filter_matrix=TRUE?")
    }
    if(nrow(ts_coord)<ncol(object)){
        object <- object[,rownames(ts_coord)]
    }

    image <- image[Cells(x = object)]
    DefaultAssay(object = image) <- "Spatial"
    object[[slice]] <- image

    return(object)
}

#' @title Decontaminate spot swapping effect in spatial transcriptomics data
#'
#' @description This is the main function implementing the SpotClean method
#' for decontaminating spot swapping effect in spatial transcriptomics data.
#'
#' @param slide_obj A slide object created or inherited from
#' \code{createSlide()}, or a \code{SpatialExperiment} object created from
#' \code{SpatialExperiment::read10xVisium()}.
#'
#' @param ... Arguments passed to other methods
#'
#'
#' @return For slide object created from \code{createSlide()}, returns a
#' slide object where the decontaminated expression matrix is in the
#' "decont" assay slot and the contamination statistics are in
#' metadata slots. Contamination statistics include ambient RNA contamination
#' (ARC) score, bleeding rate, distal rate, contamination radius,
#' contamination kernel weight matrix, log-likelihood value in each iteration,
#' estimated proportion of contamination in each tissue spot in observed data.
#' Since decontaminated and raw data have different number of columns, they can
#' not be stored in a single object.
#'
#' For \code{SpatialExperiment} object created from
#' \code{SpatialExperiment::read10xVisium()}, returns a
#' \code{SpatialExperiment} object where the decontaminated expression matrix
#' is in the "decont" assay slot and the contamination statistics are in
#' metadata slots. Raw expression matrix is also stored in the "counts" assay
#' slot. Genes are filtered based on \code{gene_cutoff}.
#'
#' @details Briefly, the contamination level for the slide is estimated based on
#' the total counts of all spots. UMI counts travelling around the slide are
#' assumed to follow Poisson distributions and modeled by a mixture of
#' Gaussian (proximal) and uniform (distal) kernels. The underlying
#' uncontaminated gene expressions are estimated by EM algorithm to
#' maximize the data likelihood. Detailed derivation can be found in our
#' manuscript.
#'
#'
#' @examples
#'
#' data(mbrain_raw)
#' spatial_dir <- system.file(file.path("extdata",
#'                                      "V1_Adult_Mouse_Brain_spatial"),
#'                            package = "SpotClean")
#' mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
#'                                        "tissue_positions_list.csv"),
#'              tissue_img_file = file.path(spatial_dir,
#'                                        "tissue_lowres_image.png"),
#'              scale_factor_file = file.path(spatial_dir,
#'                                        "scalefactors_json.json"))
#' mbrain_obj <- createSlide(mbrain_raw,
#'                           mbrain_slide_info)
#'
#' mbrain_decont_obj <- spotclean(mbrain_obj, tol=10, candidate_radius=20)
#' mbrain_decont_obj


#' @rdname spotclean
#'
#' @export

spotclean <- function(slide_obj, ...) {
    if(!class(slide_obj)%in%c("SummarizedExperiment","SpatialExperiment")){
        stop("Invalid slide object.")
    }
    UseMethod(generic = "spotclean", object = slide_obj)
}

#' @param gene_keep (vector of chr) Gene names to keep for decontamination.
#' We recommend not decontaminating lowly expressed and lowly variable genes
#' in order to save computation time. Even if user include them, their
#' decontaminated expressions will not change too much from raw expressions.
#' When setting to \code{NULL}, \code{keepHighGene()} will be
#' automatically called to filter out lowly expressed and lowly variable genes
#' before decontamination.
#' Default: \code{NULL}.
#'
#' @param maxit (int) Maximum iteration for EM parameter updates. Default: 30.
#'
#' @param tol (num) Tolerance to define convergence in EM parameter updates.
#' When the element-wise maximum difference between current and updated
#' parameter matrix is less than \code{tol}, parameters are considered
#' converged. Default: 1
#'
#' @param candidate_radius (vector of num) Candidate contamination radius.
#' A series of radius to try when estimating contamination parameters.
#' Default: \code{c(5, 10, 15, 20, 25, 30)}
#'
#' @param kernel (chr): name of kernel to use to model local contamination.
#' Supports "gaussian", "linear", "laplace", "cauchy". Default: "gaussian".
#'
#' @param verbose (logical) Whether print progress information.
#' Default: \code{TRUE}

#' @import Matrix
#' @importFrom SummarizedExperiment assays colData SummarizedExperiment assays<-
#' @importMethodsFrom S4Vectors metadata metadata<-
#' @importFrom utils txtProgressBar setTxtProgressBar
#' @importFrom dplyr filter select rename
#' @importFrom stats dist quantile optim cor lm coef
#' @importFrom methods as
#' @importFrom SpatialExperiment scaleFactors spatialCoords
#' @importFrom rlang .data
#'
#' @method spotclean SummarizedExperiment
#' @rdname spotclean
#'
#' @export

spotclean.SummarizedExperiment <- function(slide_obj, gene_keep=NULL,
                                           maxit=30, tol=1,
                                           candidate_radius=5*seq_len(6),
                                           kernel="gaussian",
                                           verbose=TRUE, ...){

    # validate arguments
    .check_arguments(gene_keep, gene_cutoff=Inf, maxit, tol,
                     candidate_radius, kernel, verbose)

    raw_data <- assays(slide_obj)$raw  # raw data matrix
    if(is.null(raw_data)){
        stop("Cannot find raw data in input slide object.")
    }
    slide <- metadata(slide_obj)$slide  # slide info

    # run SpotClean
    res <- .SpotClean(raw_data=raw_data, slide=slide,
                      gene_keep=gene_keep,
                      maxit=maxit, tol=tol,
                      candidate_radius=candidate_radius,
                      kernel=kernel,
                      verbose=verbose)

    # create output
    metadata(slide_obj)$slide <- slide[slide$tissue==1,]
    decont_obj <- createSlide(res$decont,
                              c(metadata(slide_obj),res$meta),
                              gene_cutoff = 0, verbose=FALSE)
    names(decont_obj@assays) <- "decont"

    return(decont_obj)
}

#' @param gene_cutoff (num) Filter out genes with average expressions
#' among tissue spots below or equal to this cutoff.
#' Only applies to \code{SpatialExperiment} object.
#' Default: 0.1.
#'
#' @method spotclean SpatialExperiment
#' @rdname spotclean
#'
#' @export

spotclean.SpatialExperiment <- function(slide_obj, gene_keep=NULL,
                                        gene_cutoff=0.1,
                                        maxit=30, tol=1,
                                        candidate_radius=5*seq_len(6),
                                        kernel="gaussian",
                                        verbose=TRUE, ...){
    # validate arguments
    .check_arguments(gene_keep, gene_cutoff, maxit, tol,
                     candidate_radius, kernel, verbose)

    # raw data matrix
    raw_data <- as(object = assays(slide_obj)$counts, Class = 'CsparseMatrix')
    if(is.null(raw_data)){
        stop("Cannot find raw data in input slide object.")
    }

    # collect spot info from the SpatialExperiment object
    slide <- data.frame(colData(slide_obj))
    slide <- rename(slide, tissue="in_tissue", row="array_row", col="array_col")
    slide$barcode <- rownames(slide)
    slide$tissue <- factor(as.integer(slide$tissue))
    image_pos <- spatialCoords(slide_obj)*scaleFactors(slide_obj)
    slide$imagecol <- image_pos[,"pxl_col_in_fullres"]
    slide$imagerow <- image_pos[,"pxl_row_in_fullres"]

    # filter genes
    gene_cutoff <- max(gene_cutoff,0)
    good_gene <- rowMeans(raw_data[,slide$tissue==1])>gene_cutoff
    if(verbose){
        message("Filtered out ",sum(!good_gene)
                ," genes with average expressions below or equal to ",
                gene_cutoff, ".")
    }
    raw_data <- raw_data[good_gene,]

    # run SpotClean
    res <- .SpotClean(raw_data=raw_data, slide=slide,
                      gene_keep=gene_keep,
                      maxit=maxit, tol=tol,
                      candidate_radius=candidate_radius,
                      kernel=kernel,
                      verbose=verbose)

    # create output
    decont_obj <- slide_obj[rownames(res$decont),
                            slide$barcode[slide$tissue==1]]
    assays(decont_obj)$decont <- res$decont
    metadata(decont_obj) <- c(metadata(decont_obj), res$meta)
    return(decont_obj)
}

.check_arguments <- function(gene_keep,
                             gene_cutoff,
                             maxit, tol,
                             candidate_radius,
                             kernel,
                             verbose){
    bad_args <- c()
    if(!(is.null(gene_keep) | is.character(gene_keep))){
        bad_args <- c(bad_args, "gene_keep")
    }
    if(!is.numeric(gene_cutoff)){
        bad_args <- c(bad_args, "gene_cutoff")
    }
    if(!is.numeric(maxit)) {
        bad_args <- c(bad_args, "maxit")
    }
    if(!is.numeric(candidate_radius)){
        bad_args <- c(bad_args, "candidate_radius")
    }
    if(!kernel%in%c("gaussian", "linear", "laplace", "cauchy")){
        bad_args <- c(bad_args, "kernel")
    }
    if(!is.logical(verbose)){
        bad_args <- c(bad_args, "verbose")
    }
    if(length(bad_args)>0){
        stop("invalid argument(s) (",paste(bad_args,collapse = ", "),")")
    }

    if(gene_cutoff<0) {
        stop("gene_cutoff should be non-negative")
    }
    if(maxit<=1 | maxit%%1!=0) {
        stop("maxit should be an integer greater than 1")
    }
    if(!all(candidate_radius>0)){
        stop("candidate_radius should be positive")
    }
}

.SpotClean <- function(raw_data, slide,
                       gene_keep=NULL,
                       maxit=30, tol=1,
                       candidate_radius=5*seq_len(6),
                       kernel="gaussian",
                       verbose=TRUE){

    if(verbose){
        message(Sys.time(), " Start.")
    }

    #############################
    # Step 0: setup
    #############################

    # some universal variables
    n_spots <- ncol(raw_data)  # number of spots
    ts_idx <- which(slide$tissue==1)  # tissue index
    raw_ts_data <- raw_data[,ts_idx, drop=FALSE]  # raw tissue data matrix

    # Euclidean distance matrix for spots
    slide_distance <- .calculate_euclidean_weight(
        select(slide, "imagerow", "imagecol")
    )
    rownames(slide_distance) <- colnames(slide_distance) <- slide$barcode

    # some constant matrices
    I1_y <- matrix(1,n_spots,length(ts_idx))
    I1_yy <- matrix(1,length(ts_idx),length(ts_idx))
    I_yy <- diag(length(ts_idx))

    # calculate ARC score
    ARC_score <- arcScore(raw_data,
                          background_bcs=slide$barcode[slide$tissue==0])

    # estimate spot distance in pixels
    # Note: imagecol may not correspond to col, but correspond to row
    # when the image is rotated
    if(abs(cor(slide$imagecol, slide$col))>0.99){
        lm_tmp <- lm(slide$imagecol ~ slide$col)
    }else{
        lm_tmp <- lm(slide$imagerow ~ slide$col)
    }
    spot_distance <- abs(coef(lm_tmp)[2])

    # Keep highly expressed and highly variable genes
    if(is.null(gene_keep)){
        gene_keep <- keepHighGene(raw_ts_data, verbose=verbose)

    }else{
        gene_keep <- intersect(gene_keep, rownames(raw_data))
        if(length(gene_keep)==0){
            stop("Specified genes not found in raw data.")
        }
    }

    #############################
    # Step 1: Estimate contamination parameters
    #############################

    total_counts <- colSums(raw_data[gene_keep,])
    cont_out <- c()

    if(verbose){
        message(Sys.time(), " Estimating contamination parameters...")
        pb <- txtProgressBar(max = length(candidate_radius),
                             style = 3, file = stderr())
    }

    for(i in seq_along(candidate_radius)){

        slide_weight <- .local_kernel(
            slide_distance,
            .points_to_sdv(candidate_radius[i], spot_distance),
            kernel
        )
        W_y <- t(slide_weight[ts_idx,]/rowSums(slide_weight[ts_idx,]))
        W_yy <- unname(W_y[ts_idx,])
        Wyy_tWyy <- W_yy+t(W_yy)
        WtW <- unname(crossprod(W_y))

        cont_out <- c(cont_out, .estimate_cont(obs_exp=total_counts,
                                               ts_idx=ts_idx,
                                               n_spots=n_spots,
                                               W_y=W_y, W_yy=W_yy, WtW=WtW,
                                               Wyy_tWyy=Wyy_tWyy, I_yy=I_yy,
                                               I1_y=I1_y, I1_yy=I1_yy))

        if(verbose){
            setTxtProgressBar(pb, i)
        }
    }
    gc()

    # Find the best solution
    best_radius <- which.min(unlist(lapply(cont_out,function(x) x$value)))
    bleed_rate <- cont_out[[best_radius]]$par[1]
    distal_rate <- cont_out[[best_radius]]$par[2]
    cont_radius <- candidate_radius[best_radius]

    # contamination weight matrix
    slide_weight <- .local_kernel(slide_distance,
                                  .points_to_sdv(cont_radius, spot_distance),
                                  kernel)
    slide_weight <- slide_weight[ts_idx,]/rowSums(slide_weight[ts_idx,])
    weight_mat <- distal_rate/n_spots+(1-distal_rate)*slide_weight

    #############################
    # Step 2: Decontaminate expression matrix
    #############################

    if(verbose){
        message("\n",Sys.time()," Decontaminating genes ...")
    }

    scale_factor <- rowSums(raw_data[gene_keep,])/
        rowSums(raw_ts_data[gene_keep,])
    decont_data_init <- raw_ts_data[gene_keep,]*scale_factor

    decont_out <- .decont_EM(raw_data=raw_data[gene_keep,],
                             decont_data_init = decont_data_init,
                             bleed_rate=bleed_rate,
                             distal_rate=distal_rate,
                             slide_weight = slide_weight,
                             ts_idx = ts_idx,
                             maxit=maxit,
                             tol=tol,
                             verbose=verbose)

    if(verbose){
        message("\n",Sys.time()," Scaling genes...")
    }

    decont_data <- decont_out$decont_data

    # scale up the rest genes
    gene_notkeep <- setdiff(rownames(raw_data),gene_keep)
    if(length(gene_notkeep)>0){

        scale_ratio <- rowSums(raw_data[gene_notkeep,, drop=FALSE])/
            rowSums(raw_ts_data[gene_notkeep,, drop=FALSE])
        scale_ratio[is.na(scale_ratio)] <- 1 # 0/0=NaN
        decont_data <- rbind(decont_data,
                             raw_ts_data[gene_notkeep,, drop=FALSE]*scale_ratio)
    }

    decont_data <- as(decont_data[rownames(raw_data),],"CsparseMatrix")

    # Calculation contamination rate in each tissue spot
    decont_total_counts <- colSums(decont_data)
    cont_rate <- .calculate_cont_rate(decont_total_counts,
                                      bleed_rate, distal_rate,
                                      weight_mat, n_spots)

    # write results
    meta <- list(
        bleeding_rate=bleed_rate,
        distal_rate=distal_rate,
        contamination_radius=cont_radius,
        weight_matrix=weight_mat,
        loglh=decont_out$loglh,
        decontaminated_genes=gene_keep,
        contamination_rate=cont_rate,
        ARC_score=ARC_score
    )

    if(verbose){
        message(Sys.time(), " All finished.")
    }

    return(list(decont=decont_data, meta=meta))

}

.calculate_euclidean_weight <- function(x){
    # Calculate Gaussian distances matrix of a set of 2-d points
    # Args:
    #   x (matrix of num): Each row is the coordinates of one point.
    # Returns:
    #   (matrix of num) Euclidean distance matrix

    if(ncol(x)!=2){
        stop("Incorrect dimension of input coordinates.")
    }

    # Calculate Euclidean distances
    d_mat <- dist(x, method = "euclidean", diag = TRUE)
    d_mat <- as.matrix(d_mat)
    return(d_mat)
}


.gaussian_kernel <- function(x, sigma){
    # Gaussian kernel
    # Args:
    #   x (num): euclidean distance
    #   sigma (num): bandwidth
    # Returns:
    #   (num) Gaussian distance

    exp(x^2/(-2*sigma^2))
}

.linear_kernel <- function(x, sigma){
    # linear kernel
    # Args:
    #   x (num): euclidean distance
    #   sigma (num): nonzero radius
    # Returns:
    #   (num) linear weights
    abs(sigma-pmin(x,sigma))
}

.laplace_kernel <- function(x, sigma){
    # Laplac kernel
    # Args:
    #   x (num): euclidean distance
    #   sigma (num): bandwidth
    # Returns:
    #   (num) Laplace weights
    exp(-abs(x)/sigma)
}

.cauchy_kernel <- function(x, sigma){
    # Cauchy kernel
    # Args:
    #   x (num): euclidean distance
    #   sigma (num): bandwidth
    # Returns:
    #   (num) Cauchy weights
    1/(1+x^2/sigma^2)
}

.kernel_list <- list(gaussian=.gaussian_kernel,
                     linear=.linear_kernel,
                     laplace=.laplace_kernel,
                     cauchy=.cauchy_kernel)

.local_kernel <- function(x, sigma, kernel){
    if(kernel=="linear"){
        # Linear kernel has different contamination radius
        # calculation from Gaussian,etc.
        sigma <- 2*sigma
    }
    .kernel_list[[kernel]](x,sigma)
}

.points_to_sdv <- function(cont_radius, d){
    # Transform points radius to standard deviation (the bandwidth) in
    # Gaussian kernel. This bandwidth assures that around 95% of bled-out
    # expressions are within the circle of specified spots radius
    # Args:
    #   cont_radius (int): number of points as radius where 95% contamination
    #       goes into
    #   d (num): pixel distance between two adjacent spots in a same row
    # Returns:
    #   (num): standard deviation in the Gaussian weight function

    r <- cont_radius*d # radius in pixels
    return(r/2) # sigma=r/2 -> radius=2*standard deviation -> mean +- 2SD: 95%
}

.decont_EM <- function(raw_data, decont_data_init,
                       bleed_rate, distal_rate,
                       slide_weight, ts_idx,
                       maxit=30, tol=1,
                       verbose=TRUE){
    # EM parameter updates
    # Args:
    #   raw_data (matrix of num): raw expression matrix
    #   decont_data_init (matrix of num): initial value of decontaminated
    #       expression matrix
    #   bleed_rate (num): bleeding rate
    #   distal_rate (num): distal rate
    #   slide_weight (matrix of num): Gaussian weight matrix
    #   ts_idx (vector of int): indices of tissue spots
    #   maxit (int): maximum number of iteration
    #   tol (num): tolerance for convergence
    #   verbose (logical): whether print progress messages
    # Returns:
    #   a list containing decontaminated expression matrix and a series
    #   of log-likelihood values during iteration


    # Initialization
    decont_data <- decont_data_init
    raw_ts_data <- raw_data[,ts_idx]

    Loglh <- c()
    n_it <- 1
    N_gene <- nrow(raw_data)
    N_spot <- ncol(raw_data)
    N_ts_spot <- length(ts_idx)
    bleed_weight_mat <- bleed_rate*(
        distal_rate/N_spot+(1-distal_rate)*slide_weight
    )

    # Iteration
    repeat{
        if(verbose){
            message("\nIteration: ",n_it)
        }

        # E-step

        Eta <- decont_data%*%bleed_weight_mat
        Eta[,ts_idx] <- Eta[,ts_idx]+decont_data*(1-bleed_rate)

        # stayed
        S_ <- (1-bleed_rate)*raw_ts_data*decont_data/Eta[,ts_idx]
        # contamination
        C_ <- (raw_data/Eta)%*%t(slide_weight)*
            decont_data*bleed_rate*(1-distal_rate)+
            decont_data*bleed_rate*distal_rate/N_spot *
            rowSums(raw_data/Eta)


        # M-step

        decont_data_new <- S_+C_
        loglh <- sum(raw_data*log(Eta)-Eta, na.rm = TRUE)

        # Difference between two adjacent iterations
        Lambda_maxdiff <- max(abs(decont_data_new-decont_data))

        decont_data <- decont_data_new
        Loglh <- c(Loglh, loglh)

        if(verbose){
            message("Log-likelihood: ",round(loglh,3),
                    "\nMax difference of decontaminated expressions: ",
                    round(Lambda_maxdiff,3))
        }

        if(n_it>1){
            if(Lambda_maxdiff < tol){
                if(verbose) message("Parameter converged.")
                break
            }else if(n_it>=maxit){
                if(verbose) message("Reached maximum iteration.")
                break
            }
        }

        n_it <- n_it+1

    }

    return(list(decont_data=decont_data,
                loglh=Loglh))
}


.estimate_cont <- function(obs_exp, ts_idx, n_spots,
                           W_y, W_yy, WtW, Wyy_tWyy, I_yy, I1_y, I1_yy){
    # Estimate contamination parameters using graident descent
    # Args:
    #   obs_exp (vector of num): observed expressions for all spots
    #   ts_idx (vector of int): indices of tissue spots
    #   n_spots (int): total number of spots
    #   W_y (matrix of num): contamination weight matrix
    #   W_yy (matrix of num): contamination weight matrix from tissue to tissue
    #   WtW (matrix of num): W_transpose dot-product W
    #   Wyy_tWyy (matrix of num): Wyy_transpose dot-product Wyy
    #   I_yy (matrix of num): diagonal matrix of tissue spots
    #   I1_y (matrix of num): matrix of ones
    #   I1_yy (matrix of num): matrix of ones for tissue spots
    # Returns:
    #   a list of optimization outputs from optim()

    bg_idx <- setdiff(seq_along(obs_exp),ts_idx)
    nonzero_pos <- which(obs_exp[ts_idx]>0)

    # estimate initial values
    bleed_rate_init <- sum(obs_exp[bg_idx])/length(bg_idx)*
        n_spots/sum(obs_exp)

    bg_quant <- quantile(obs_exp[bg_idx], c(0.25, 0.5))
    bg_trim <- obs_exp[bg_idx]>=bg_quant[1] & obs_exp[bg_idx]<=bg_quant[2]
    uniform_cont <- mean(obs_exp[bg_idx][bg_trim])
    distal_rate_init <- uniform_cont/sum(obs_exp[bg_idx])*length(bg_idx)
    # trim the initial value
    distal_rate_init <- min(distal_rate_init, 0.5)

    mu_init <- obs_exp[ts_idx][nonzero_pos]
    mu_init <- mu_init/sum(mu_init)*sum(obs_exp)

    # assign parameter bounds
    lower_bounds <- rep(0,length(mu_init)+2)
    lower_bounds[c(1,2)] <- c(sum(obs_exp[bg_idx])/sum(obs_exp),0.1)
    upper_bounds <- rep(Inf,length(lower_bounds))
    upper_bounds[c(1,2)] <- 1

    # calculate other matrices
    I1tZ <- crossprod(I1_y,obs_exp)
    WtZ <- crossprod(W_y,obs_exp)

    # other candidate initial values
    cand_init <- list(pmax(c(bleed_rate_init,distal_rate_init),0.1),
                      c(0.3,0.3),c(0.5,0.3))

    # minimize RSS using L-BFGS-B
    opt_list <- list()
    for(init in seq_along(cand_init)){
        x_init <- unname(c(cand_init[[init]],mu_init))
        opt_list[[init]] <- optim(x_init, .fn_optim, .gr_optim,
                                  method = "L-BFGS-B",
                                  obs_exp=obs_exp, ts_idx=ts_idx,
                                  nonzero_pos = nonzero_pos, n_spots=n_spots,
                                  W_yy=W_yy, WtW=WtW,
                                  Wyy_tWyy=Wyy_tWyy,I_yy=I_yy,
                                  I1_yy=I1_yy, WtZ=WtZ, I1tZ=I1tZ,
                                  lower=lower_bounds,
                                  upper=upper_bounds,
                                  control = list(maxit=100))
    }

    best_par <- which.min(unlist(lapply(opt_list, function(x) x$value)))
    opt <- opt_list[[best_par]]

    x_final <- numeric(length(ts_idx))
    x_final[nonzero_pos] <- opt$par[-c(1,2)]
    opt$par <- c(opt$par[c(1,2)],x_final)

    return(list(opt))
}

.calculate_cont_rate <- function(decont_total_counts,
                                 bleed_rate, distal_rate,
                                 weight_mat, n_spots){
    # Estimate proportion of contamination in each tissue spot in observed data

    # expression originated from the spot = non-bled expression +
    # contamination going to itself

    stayed_counts <- decont_total_counts*(1-bleed_rate)+
        decont_total_counts*bleed_rate*
        diag(weight_mat[names(decont_total_counts),
                        names(decont_total_counts)])

    # fitted total expression = stayed + received
    fitted_total_counts <- decont_total_counts*(1-bleed_rate)+
        ((decont_total_counts*bleed_rate)%*%
             weight_mat)[,names(decont_total_counts)]

    received_counts <- fitted_total_counts-stayed_counts

    cont_rate <- received_counts/fitted_total_counts

    return(cont_rate)
}

.fn_optim <- function(x, obs_exp, ts_idx, nonzero_pos,
                      W_yy, WtW, I_yy, Wyy_tWyy, I1_yy, n_spots, WtZ, I1tZ){
    # objective function: residual sum of squares

    x_coef <- x[-c(1,2)]
    x_c_rate <- x[1] # bleed_rate
    x_g_rate <- x[2] # distal_rate

    x_coef%*%.AtA(x_c_rate, x_g_rate,
                  WtW[nonzero_pos,nonzero_pos],
                  I_yy[nonzero_pos,nonzero_pos],
                  Wyy_tWyy[nonzero_pos,nonzero_pos],
                  I1_yy[nonzero_pos,nonzero_pos],
                  n_spots)%*%x_coef-
        2*crossprod(.AtZ(x_c_rate, x_g_rate,
                         WtZ[nonzero_pos], I1tZ[nonzero_pos],
                         obs_exp[ts_idx[nonzero_pos]],n_spots),
                    x_coef)


}


.gr_optim <- function(x, obs_exp, ts_idx, nonzero_pos,
                      W_yy, WtW, I_yy, Wyy_tWyy, I1_yy, n_spots, WtZ, I1tZ){
    # Gradient of .fn_optim

    # recover full dimension of x
    N <- length(ts_idx)
    x_coef <- numeric(N)
    x_coef[nonzero_pos] <- x[-c(1,2)]
    x_c_rate <- x[1] # bleed_rate
    x_g_rate <- x[2] # distal_rate


    x_coef1 <- x[-c(1,2)]
    x_c_rate <- x[1] # bleed_rate
    x_g_rate <- x[2] # distal_rate


    # gradient of spots expressions
    g_coef <- 2*crossprod(.AtA(x_c_rate, x_g_rate,
                               WtW[nonzero_pos,nonzero_pos],
                               I_yy[nonzero_pos,nonzero_pos],
                               Wyy_tWyy[nonzero_pos,nonzero_pos],
                               I1_yy[nonzero_pos,nonzero_pos],
                               n_spots),x_coef1)-
        2*.AtZ(x_c_rate, x_g_rate, WtZ[nonzero_pos], I1tZ[nonzero_pos],
               obs_exp[ts_idx[nonzero_pos]], n_spots)


    # gradient of bleeding rate
    g_c_rate <- x_coef1%*%.dAtA_dr(x_c_rate, x_g_rate,
                                   WtW[nonzero_pos,nonzero_pos],
                                   I_yy[nonzero_pos,nonzero_pos],
                                   Wyy_tWyy[nonzero_pos,nonzero_pos],
                                   I1_yy[nonzero_pos,nonzero_pos],
                                   n_spots) %*%x_coef1-
        2*crossprod(.dAtZ_dr(x_c_rate, x_g_rate, WtZ[nonzero_pos],
                             I1tZ[nonzero_pos], n_spots,
                             obs_exp[ts_idx[nonzero_pos]]),x_coef1)

    # gradient of distal rate
    g_g_rate <- x_coef1%*%.dAtA_dc(x_c_rate, x_g_rate,
                                   WtW[nonzero_pos,nonzero_pos],
                                   Wyy_tWyy[nonzero_pos,nonzero_pos],
                                   n_spots,
                                   I1_yy[nonzero_pos,nonzero_pos]) %*%x_coef1-
        2*crossprod(.dAtZ_dc(x_c_rate, WtZ[nonzero_pos],
                             I1tZ[nonzero_pos],
                             n_spots),x_coef1)

    c(g_c_rate,g_g_rate,g_coef)
}


# below are other internal functions for gradient calculation

.AtA <- function(bleed_rate, distal_rate, WtW,
                 I_yy, Wyy_tWyy,I1_yy, n_spots){
    (1-bleed_rate)^2*I_yy+
        bleed_rate^2*(1-distal_rate)^2*WtW+
        bleed_rate*(1-bleed_rate)*(1-distal_rate)*Wyy_tWyy+
        bleed_rate*distal_rate*(2-bleed_rate*distal_rate)/n_spots*I1_yy
}

.AtZ <- function(bleed_rate, distal_rate, WtZ, I1tZ, obs_ts_exp, n_spots){
    (1-bleed_rate)*obs_ts_exp+bleed_rate*(1-distal_rate)*WtZ+
        bleed_rate*distal_rate/n_spots*I1tZ
}

.dAtA_dr <- function(bleed_rate, distal_rate,
                     WtW, I_yy, Wyy_tWyy,I1_yy, n_spots){
    2*(bleed_rate-1)*I_yy+2*bleed_rate*(1-distal_rate)^2*WtW+
        (1-2*bleed_rate)*(1-distal_rate)*Wyy_tWyy+
        2*(distal_rate-bleed_rate*distal_rate^2)/n_spots*I1_yy
}

.dAtZ_dr <- function(bleed_rate, distal_rate,
                     WtZ, I1tZ, n_spots, obs_ts_exp){
    (1-distal_rate)*WtZ+distal_rate/n_spots*I1tZ-obs_ts_exp

}

.dAtA_dc <- function(bleed_rate, distal_rate,
                     WtW, Wyy_tWyy, n_spots, I1_yy){
    2*bleed_rate^2*(distal_rate-1)*WtW+
        bleed_rate*(bleed_rate-1)*Wyy_tWyy+
        2*(bleed_rate-bleed_rate^2*distal_rate)/n_spots*I1_yy
}

.dAtZ_dc <- function(bleed_rate, WtZ, I1tZ, n_spots){
    bleed_rate/n_spots*I1tZ-bleed_rate*WtZ

}

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(
    tissue_csv_file=file.path(spatial_dir,
                              "tissue_positions_list.csv"),
    tissue_img_file = file.path(spatial_dir,
                                "tissue_lowres_image.png"),
    scale_factor_file = file.path(spatial_dir,
                                  "scalefactors_json.json"))

mbrain_obj <- createSlide(mbrain_raw,
                          mbrain_slide_info)

# test output

test_that("Decontamination",{
    expect_no_error(mbrain_decont_obj <- spotclean(
        mbrain_obj, candidate_radius=20, maxit = 3, verbose = FALSE))
    expect_s4_class(mbrain_decont_obj,"SummarizedExperiment")
    expect_identical(names(mbrain_decont_obj@assays),"decont")
    expect_equal(metadata(mbrain_decont_obj)$ARC_score, 0.05160659)
    expect_equal(metadata(mbrain_decont_obj)$bleeding_rate, 0.3301657,
                 tolerance = 4e-8)
    expect_equal(metadata(mbrain_decont_obj)$distal_rate, 0.2154095,
                 tolerance = 4e-8)
    expect_equal(metadata(mbrain_decont_obj)$weight_matrix[
        "AAACAAGTATCTCCCA-1","AAACAACGAATAGTTC-1"
    ], 0.00004315095, tolerance = 4e-8)
    expect_equal(metadata(mbrain_decont_obj)$loglh,
                 c(167030816, 167314060, 167379802))
    expect_equal(unname(metadata(mbrain_decont_obj)$contamination_rate[
        c("AAACAAGTATCTCCCA-1", "AAACAATCTACTAGCA-1", "AAACACCAATAACTGC-1")
        ]), c(0.2794837, 0.3102042, 0.2128133), tolerance = 4e-8)

    expect_equal(rowSums(assay(mbrain_decont_obj)),rowSums(assay(mbrain_obj)))
    expect_equal(assay(mbrain_decont_obj)["Bc1","AAACAAGTATCTCCCA-1"],
                 1457.840109)

})


# test internal computations

test_that("Internal computations", {
    small_mat <- matrix(1:4,2,2)
    small_edist <- .calculate_euclidean_weight(small_mat)
    small_gdist <- .gaussian_kernel(small_edist,10)
    small_ldist <- .linear_kernel(small_edist,20)
    small_lapdist <- .laplace_kernel(small_edist,30)
    small_cdist <- .cauchy_kernel(small_edist,40)

    expect_equal(small_edist[1,2],sqrt(2))
    expect_equal(small_edist,t(small_edist))
    expect_equal(small_gdist[1,2],0.99004984)
    expect_equal(small_ldist[1,2],18.5857865)
    expect_equal(small_lapdist[1,2],0.9539534)
    expect_equal(small_cdist[1,2],0.99875156)

    expect_equal(.points_to_sdv(10,5), 25)

})

# test input parameters
test_that("Data loading", {
    expect_error(spotclean(mbrain_obj, gene_keep = "foo"),
                 "Specified genes not found")
})

names(mbrain_obj@assays) <- "decont"

test_that("Wrong assay", {
    expect_error(spotclean(mbrain_obj, gene_keep = "foo"),
                 "Cannot find raw data")
})

#' @title Calculate the ambient RNA contamination (ARC) score
#'
#' @description The ARC score is intended for subjectively measuring the level
#' of contamination caused by ambient RNAs for a given spatial transcriptomics
#' or droplet-based single-cell RNA-seq data. Intuitively, this score is a lower
#' bound of the average proportion of contaminated expressions in tissue
#' spots or cell droplets in the observed contaminated data.
#'
#' ARC score is calculated as follows: (1) estimate the average amount of
#' contaminated UMI counts received per spot (droplet) using the total UMI
#' counts in background spots (empty droplets) divided by total number of
#' spots (droplets). This is an underestimation as it does not account for
#' contamination inside tissue spots or cell droplets. (2) estimate the average
#' amount of observed UMI counts per tissue spot (cell droplet) using the
#' total UMI counts in tissue spots (cell droplets) divided by number of
#' tissue spots (cell droplets). (3) Divide the value in (1) by the value in
#' (2) to get the ARC score.
#'
#' This lower bound is very conservative as it only
#' accounts for observable contamination in background spots or empty droplets,
#' neglecting contamination in tissue spots or cell droplets. However, it
#' is totally subjective, not relying on any complicated assumptions and
#' modelling. ARC score serves as a relative contamination measurement for
#' comparison among different datasets and protocols.
#'
#' @param object A gene-by-barcode count matrix or a slide object created or
#' inherited from \code{createSlide()}.
#' Ideally this should give the raw count matrix with all genes and barcodes
#' without any filtering.
#'
#' @param ... Arguments passed to other methods
#'
#' @return (num) The ARC score of given data.
#'
#' @examples
#'
#' data(mbrain_raw)
#' spatial_dir <- system.file(file.path("extdata",
#'                                      "V1_Adult_Mouse_Brain_spatial"),
#'                            package = "SpotClean")
#' mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
#'                                        "tissue_positions_list.csv"),
#'              tissue_img_file = file.path(spatial_dir,
#'                                        "tissue_lowres_image.png"),
#'              scale_factor_file = file.path(spatial_dir,
#'                                        "scalefactors_json.json"))
#'
#' background_bcs <- dplyr::filter(mbrain_slide_info$slide, tissue==0)$barcode
#' arcScore(mbrain_raw, background_bcs)
#'
#' mbrain_obj <- createSlide(mbrain_raw, mbrain_slide_info)
#' arcScore(mbrain_obj)
#'
#' @rdname arcScore
#'
#' @export

arcScore <- function(object, ...) {
    UseMethod(generic = "arcScore", object = object)
}


#' @param background_bcs (vector of chr) Background barcodes in
#' \code{count_mat}. For spatial data, these are background spots not
#' covered by tissue. For single-cell data, these are empty droplets not
#' containing cells.
#'
#' @importFrom Matrix colSums
#' @importFrom methods as
#'
#' @method arcScore default
#' @rdname arcScore
#' @export
#'
arcScore.default <- function(object, background_bcs, ...){

    if (!inherits(x = object, 'Matrix')) {
        object <- as(object = as.matrix(x = object), Class = 'Matrix')
    }
    if (!inherits(x = object, what = 'dgCMatrix')) {
        object <- as(object = object, Class = 'CsparseMatrix')
    }

    if(!all(background_bcs%in%colnames(object))){
        stop("Invalid background barcodes.")
    }

    total_counts <- colSums(object)
    # underestimated contamination per barcode
    cont <- sum(total_counts[background_bcs])/ncol(object)
    # expression per tissue/cell barcode
    ts_bcs <- setdiff(colnames(object),background_bcs)
    expr <- mean(total_counts[ts_bcs])
    # proportion
    arc <- cont/expr

    return(arc)
}


#' @method arcScore SummarizedExperiment
#' @rdname arcScore
#'
#' @importFrom SummarizedExperiment assay
#' @importFrom S4Vectors metadata
#' @importFrom dplyr filter
#' @importFrom rlang .data
#'
#' @export
#'
arcScore.SummarizedExperiment <- function(object, ...){

    background_bcs <- filter(metadata(object)$slide, .data$tissue==0)$barcode
    count_mat <- assay(object)

    total_counts <- colSums(count_mat)
    # underestimated contamination per barcode
    cont <- sum(total_counts[background_bcs])/ncol(count_mat)
    # expression per tissue/cell barcode
    ts_bcs <- setdiff(colnames(count_mat),background_bcs)
    expr <- mean(total_counts[ts_bcs])
    # proportion
    arc <- cont/expr

    return(arc)
}

#' @title Visualize spot values on the 2D slide
#'
#' @description Generate and visualize spot labels on the 2D slide
#' as a ggplot2 object. Spot labels can be given via parameter \code{label} as
#' external input or one column in the \code{slide} slot of the slide
#' object's metadata via \code{label_col}. Exactly one of \code{label}
#' and \code{label_col} must be specified.
#'
#' @param object A slide object created or
#' inherited from \code{createSlide()}, or a \code{data.frame} of slide
#' information with columns: barcodes, tissue, imagerow, imagecol, etc.
#'
#' @param ... Arguments passed to other methods
#'
#' @return A \code{ggplot2} object.
#'
#' @examples
#'
#' data(mbrain_raw)
#' spatial_dir <- system.file(file.path("extdata",
#'                                      "V1_Adult_Mouse_Brain_spatial"),
#'                            package = "SpotClean")
#' mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
#'                                        "tissue_positions_list.csv"),
#'              tissue_img_file = file.path(spatial_dir,
#'                                        "tissue_lowres_image.png"),
#'              scale_factor_file = file.path(spatial_dir,
#'                                        "scalefactors_json.json"))
#' mbrain_obj <- createSlide(mbrain_raw,
#'                           mbrain_slide_info)
#'
#' gp <- visualizeHeatmap(mbrain_obj, "Bc1",
#'                      title="mbrain", legend_title="Bc1 expression")
#' plot(gp)


#' @rdname visualizeHeatmap
#'
#' @export

visualizeHeatmap <- function(object, ...) {
    UseMethod(generic = "visualizeHeatmap", object = object)
}


#' @param value (numeric vector or chr) Either a vector of numeric values for
#' all spots, or the name of one gene in \code{exp_matrix} or in the expression
#' matrix in the slide object.
#' In the former case, the order of values in the
#' vector should match the spot barcodes in the slide object.
#'
#' @param exp_matrix (matrix of num) When \code{object} is a data frame and
#' \code{value} is a single character
#' string, will search the matching gene in \code{exp_matrix} and plot the gene
#' expression. Default: \code{NULL}
#'
#' @param subset_barcodes (vector of chr) A subset of spot barcodes to plot.
#' By default it plots all spots in the slide object. This can be useful
#' when only plotting tissue spots or specific tissue types or regions.
#' Default: \code{NULL}
#'
#' @param logged (logical) Specify if the color scale is log1p transformed.
#' Default: \code{TRUE}
#'
#' @param viridis (logical) If true, color scale uses viridis.
#' Otherwise, use rainbow. Default: \code{TRUE}
#'
#' @param legend_range (length 2 vector of num) Custom legend
#' range of the value. By default uses the range of the plotted values.
#' Default: \code{NULL}
#'
#' @param title (chr) Title of the plot. Default: \code{""}
#'
#' @param legend_title (chr) Title of the legend. Under default,
#' use \code{value} as legend title. Default: \code{NULL}
#'
#' @importFrom methods as
#'
#' @method visualizeHeatmap default
#' @rdname visualizeHeatmap
#'
#' @export

visualizeHeatmap.default <- function(object, value, exp_matrix=NULL,
                                     subset_barcodes=NULL,
                                     logged=TRUE, viridis=TRUE,
                                     legend_range=NULL,
                                     title="", legend_title=NULL,
                                     ...){

    if (!inherits(x = object, "data.frame")) {
        object <- as(object = object, Class = "data.frame")
    }

    slide <- object

    gp <- .visualize_heatmap(slide=slide, value=value,
                             exp_mat=exp_matrix,
                             subset_barcodes=subset_barcodes,
                             logged=logged, viridis=viridis,
                             legend_range=legend_range,
                             title=title, legend_title=legend_title,
                             ...)
    return(gp)
}


#' @method visualizeHeatmap SummarizedExperiment
#' @rdname visualizeHeatmap
#' @importFrom SummarizedExperiment assay
#' @importMethodsFrom S4Vectors metadata
#' @export
#'
visualizeHeatmap.SummarizedExperiment <- function(object, value,
                                                  subset_barcodes=NULL,
                                                  logged=TRUE, viridis=TRUE,
                                                  legend_range=NULL,
                                                  title="", legend_title=NULL,
                                                  ...){
    slide <- metadata(object)$slide
    exp_mat <- assay(object)

    gp <- .visualize_heatmap(slide=slide, value=value,
                             exp_mat=exp_mat,
                             subset_barcodes=subset_barcodes,
                             logged=logged, viridis=viridis,
                             legend_range=legend_range,
                             title=title, legend_title=legend_title,
                             ...)
    return(gp)
}


#' @importFrom ggplot2 ggplot aes geom_point coord_cartesian xlim ylim
#' @importFrom ggplot2 xlab ylab ggtitle labs theme_set theme_bw
#' @importFrom ggplot2 theme element_blank element_line
#' @importFrom dplyr filter
#' @importFrom rlang .data
#'
.visualize_heatmap <- function(slide, value, exp_mat=NULL,
                               subset_barcodes=NULL,
                               logged=TRUE, viridis=TRUE,
                               legend_range=NULL,
                               title="", legend_title=NULL,
                               ...){

    # manipulate value to plot
    if(length(value)==1 & is.character(value)){
        if(is.null(legend_title)){
            legend_title <- value
        }

        # value to plot is in slide dataframe
        if(value%in%colnames(slide)){
            slide$value <- slide[,value]
        }else{
            # value to plot is in given matrix
            if(is.null(exp_mat)){
                stop("You must provide an input expression matrix to plot ",
                     value," expressions.")
            }
            if(!value%in%rownames(exp_mat)){
                stop("Specified gene does not exist in the expression matrix.")
            }

            # if expression matrix does not match slide info
            shared_bcs <- intersect(colnames(exp_mat), slide$barcode)
            if(length(shared_bcs)==0){
                stop("Barcodes in input matrix do not ",
                     "match any barcodes in slide.")
            }
            missed_bcs <- setdiff(slide$barcode,shared_bcs)

            value <- c(exp_mat[value,shared_bcs],rep(NA, length(missed_bcs)))
            names(value) <- c(shared_bcs, missed_bcs)
            slide$value <- value[slide$barcode]
        }

    }else if(length(value)==nrow(slide)){
        # values to plot is directly given
        slide$value <- as.numeric(value)
    }else{
        stop("Invalid value input.")
    }

    # subsetting barcodes
    if(!is.null(subset_barcodes)){
        slide <- filter(slide, .data$barcode%in%subset_barcodes)
    }

    # setup legend breaks
    if(is.null(legend_range)){
        legend_range <- c(min(0,min(slide$value, na.rm = TRUE)),
                          max(slide$value, na.rm = TRUE))
    }
    if(logged){
        legend_breaks <- expm1( log1p(min(legend_range))+
                                          diff(log1p(legend_range))/4*0:4 )
    }else{
        legend_breaks <- min(legend_range)+diff(legend_range)/4*0:4
    }
    legend_breaks[c(1,5)] <- legend_range
    if(diff(legend_range)>10){
        legend_breaks[seq_len(4)] <- ceiling(legend_breaks[seq_len(4)])
        legend_breaks[5] <- floor(legend_breaks[5])
    }

    if(min(legend_range)==0){
        slide_show <- filter(slide,value>0)
        slide_hide <- filter(slide, value==0)
    }else{
        slide_show <- filter(slide,
                             value>=min(legend_range),value<=max(legend_range))
        slide_hide <- filter(slide,
                             value<min(legend_range) | value>max(legend_range))
        if(nrow(slide_hide)>0){
            warning(nrow(slide_hide), " spots outside specified range.\n")
        }

    }

    # plot
    gp <- ggplot(slide_show, aes(x = .data$imagecol,
                                 y = .data$imagerow, fill = .data$value)) +
        geom_point(
            shape = 21,
            colour = "white",
            size = 1.75) +
        geom_point(data=slide_hide,
                   shape = 21,
                   colour = "white",
                   fill="#d6d6d6",
                   size = 1.75,
                   alpha=0.3
        ) +
        coord_cartesian(expand = FALSE) +
        .scale_fill_fun(viridis=viridis,
                        trans = ifelse(logged,"log1p","identity"),
                        breaks=legend_breaks, limits=legend_range) +
        xlim(0, max(slide$width)) +
        ylim(max(slide$height), 0) +
        xlab("") +
        ylab("") +
        ggtitle(title) +
        labs(fill = legend_title) +
        theme_set(theme_bw(base_size = 10)) +
        theme(
            panel.grid.major = element_blank(),
            panel.grid.minor = element_blank(),
            panel.background = element_blank(),
            axis.line = element_line(colour = "black"),
            axis.text = element_blank(),
            axis.ticks = element_blank()
        )

    return(gp)
}

# Color scales: viridis v.s. rainbow
#' @importFrom viridis scale_fill_viridis
#' @importFrom grDevices colorRampPalette
#' @importFrom RColorBrewer brewer.pal
#' @importFrom ggplot2 scale_fill_gradientn
#'
.scale_fill_fun <- function(viridis=TRUE, ...){
    if(viridis){
        return(scale_fill_viridis(...))
    }else{
        myPalette <- colorRampPalette(rev(brewer.pal(11, "Spectral")))
        return(
            scale_fill_gradientn(colours = myPalette(100), ...)
        )
    }
}

#' SpotClean: a computational method to adjust for spot swapping in
#' spatial transcriptomics data
#'
#' SpotClean is a computational method to adjust for spot swapping in
#' spatial transcriptomics data. Recent spatial transcriptomics experiments
#' utilize slides containing thousands of spots with spot-specific barcodes
#' that bind mRNA. Ideally, unique molecular identifiers at a spot measure
#' spot-specific expression, but this is often not the case due to bleed from
#' nearby spots, an artifact we refer to as spot swapping. SpotClean is able
#' to estimate the contamination rate in observed data and decontaminate the
#' spot swapping effect, thus increase the sensitivity and precision of
#' downstream analyses.
#'
#' To learn more about \code{SpotClean}, see the vignette
#' using \code{browseVignettes(package = "SpotClean")}.
#'
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

#' @title Create a new slide object
#'
#' @description This function takes input of the count matrix
#' (from \code{read10xRaw} or \code{read10xRawH5}) and slide information
#' (from \code{read10xSlide} or manually specified data frame) and outputs
#' a \code{SummarizedExperiment} object
#' as our slide object for downstream decontamination and visualization.
#'
#'
#' @param count_mat (matrix of num) The raw gene-by-barcode count matrix.
#' Can be either standard matrix format or sparse matrix format.
#'
#' @param slide_info (list or data.frame) A list of slide information from
#' \code{read10xSlide()}, or a data frame only containing spot information
#' like barcode, tissue, imagerow, imagecol, etc.
#'
#' @param gene_cutoff (num) Filter out genes with average expressions
#' among tissue spots below or equal to this cutoff.
#' Default: 0.1
#'
#' @param verbose (logical) Whether print progress information.
#' Default: \code{TRUE}
#'
#' @return A \code{SummarizedExperiment} object containing
#' gene expression and spot metadata.
#'
#' @examples
#'
#' data(mbrain_raw)
#' spatial_dir <- system.file(file.path("extdata",
#'                                      "V1_Adult_Mouse_Brain_spatial"),
#'                            package = "SpotClean")
#' mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
#'                                        "tissue_positions_list.csv"),
#'              tissue_img_file = file.path(spatial_dir,
#'                                        "tissue_lowres_image.png"),
#'              scale_factor_file = file.path(spatial_dir,
#'                                        "scalefactors_json.json"))
#'
#' mbrain_obj <- createSlide(mbrain_raw,
#'                           mbrain_slide_info)
#' mbrain_obj


#' @importFrom SummarizedExperiment SummarizedExperiment
#' @importFrom Matrix rowMeans
#'
#'
#' @export

createSlide <- function(count_mat, slide_info, gene_cutoff=0.1, verbose=TRUE){

    # validate arguments
    if(!is.numeric(gene_cutoff)){
        stop("invalid argument (gene_cutoff)")
    }
    if(gene_cutoff<0) {
        stop("gene_cutoff should be non-negative")
    }
    if(!is.logical(verbose)){
        stop("invalid argument (verbose)")
    }
    if(is.data.frame(slide_info) &
       all(c("barcode","tissue","imagerow","imagecol"
             )%in%colnames(slide_info))){
        warning("Input slide info does not contain image.\n")
        slide_info <- list(slide=slide_info)
    }

    rownames(slide_info$slide) <- slide_info$slide$barcode
    if(!identical(sort(colnames(count_mat)),
                 sort(slide_info$slide$barcode)
                 )){
        warning("Barcodes in count matrix do not match ",
             "barcodes in slide information.")
        
        # https://github.com/zijianni/SpotClean/issues/15
        # filter out barcodes in count matrix but not in slide info
        unique_mat_bc = !colnames(count_mat)%in%slide_info$slide$barcode
        warning("Remove ", sum(unique_mat_bc), " barcodes from count matrix.")
        count_mat = count_mat[,!unique_mat_bc]
        
        # filter out barcodes in slide info but not in count matrix
        unique_slide_bc = !slide_info$slide$barcode%in%colnames(count_mat)
        warning("Remove ", sum(unique_slide_bc), 
                " barcodes from slide information.")
        slide_info$slide = slide_info$slide[!unique_slide_bc,]
        
        if(ncol(count_mat)==0){
            stop("No barcode in count matrix matches slide information.")
        }
        
    }

    # rearrange barcodes
    slide_info$slide <- slide_info$slide[colnames(count_mat),]
    count_ts_mat <- count_mat[,slide_info$slide$tissue==1]

    # filter genes
    gene_cutoff <- max(gene_cutoff,0)
    good_gene <- rowMeans(count_ts_mat)>gene_cutoff

    if(verbose){
        message("Filtered out ",sum(!good_gene)
                ," genes with average expressions below or equal to ",
                gene_cutoff, ".")
    }

    count_mat <- count_mat[good_gene,]

    obj <- SummarizedExperiment(assays=list(raw=count_mat),
                                metadata=slide_info)
    return(obj)
}

data(mbrain_raw)

test_that("Data dimension",{
    expect_equal(dim(mbrain_raw),c(100,4992))
})

test_that("Gene and barcode",{
    expect_identical(rownames(mbrain_raw)[1],"Bc1")
    expect_identical(colnames(mbrain_raw)[999],"ATATCGTTCCTCGAAC-1")

})

test_that("Values",{
    expect_equal(mbrain_raw[10,10],14)
})

---
title: "SpotClean adjusts for spot swapping in spatial transcriptomics data"
authors: "Zijian Ni and Christina Kendziorski"
package: SpotClean
date: "`r Sys.Date()`"
output: 
    BiocStyle::html_document:
        toc: true
vignette: >
    %\VignetteIndexEntry{SpotClean}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}  
---


```{r, echo=FALSE, results="hide", message=FALSE}
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
set.seed(1)
```


# Introduction

Spatial transcriptomics (ST), named [Method of the Year 2020](https://www.nature.com/articles/s41592-020-01033-y) by *Nature Methods* in 2020, is a powerful and widely-used experimental method for profiling genome-wide gene expression across a tissue. In a typical ST experiment, fresh-frozen (or FFPE) tissue is sectioned and placed onto a slide containing spots, with each spot containing millions of capture oligonucleotides with spatial barcodes unique to that spot. The tissue is imaged, typically via Hematoxylin and Eosin (H&E) staining. Following imaging, the tissue is permeabilized to release mRNA which then binds to the capture oligonucleotides, generating a cDNA library consisting of transcripts bound by barcodes that preserve spatial information. Data from an ST experiment consists of the tissue image coupled with RNA-sequencing data collected from each spot. A first step in processing ST data is tissue detection, where spots on the slide containing tissue are distinguished from background spots without tissue. Unique molecular identifier (UMI) counts at each spot containing tissue are then used in downstream analyses.

Ideally, a gene-specific UMI at a given spot would represent expression of that gene at that spot, and spots without tissue would show no (or few) UMIs. This is not the case in practice. Messenger RNA bleed from nearby spots causes substantial contamination of UMI counts, an artifact we refer to as spot swapping. On average, we observe that more than 30% of UMIs at a tissue spot did not originate from this spot, but from other spots contaminating it. Spot swapping confounds downstream inferences including normalization, marker gene-based annotation, differential expression and cell type decomposition.

We developed **SpotClean** to adjust for the effects of spot swapping in ST experiments. SpotClean is able to measure the per-spot contamination rates in observed data and decontaminate gene expression levels, thus increases the sensitivity and precision of downstream analyses. Our package `SpotClean` is built based on 10x Visium spatial transcriptomics experiments, currently the most widely-used commercial protocol, providing functions to load raw spatial transcriptomics data from 10x Space Ranger outputs, decontaminate the spot swapping effect, estimate contamination levels, visualize expression profiles and spot labels on the slide, and connect with other widely-used packages for further analyses. SpotClean can be potentially extended to other spatial transcriptomics data as long as the gene expression data in both tissue and background regions are available. `SpotClean` is compatible with the [`SpatialExperiment`](https://bioconductor.org/packages/release/bioc/html/SpatialExperiment.html) class and supports downstream analyses via [`Seurat`](https://satijalab.org/seurat/).

As a computational tool for analyzing spatial transcriptomics data, `SpotClean` has been submitted to [Bioconductor](https://www.bioconductor.org/), making the R package well-documented and well-maintained for improved reproducibility and user experience.

# Quick Start

## Installation

Install from Bioconductor:

```{r, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("SpotClean")

```

Load package after installation:

```{r}
library(SpotClean)
library(S4Vectors)
```

## Short Demo

Here we quickly demonstrate the general SpotClean workflow on the built-in example data. A step-by-step illustration can be found in the next section.

Note: codes in this chunk are only for illustrative purpose and are not runnable. Runnable codes can be found in the next section.

```{r, eval=FALSE}
# Not run

# Load 10x Visium data
mbrain_raw <- read10xRaw("/path/to/matrix/folder")
mbrain_slide_info <- read10xSlide("/path/to/tissue/csv", 
                                  "/path/to/tissue/image", 
                                  "/path/to/scale/factor")

# Visualize raw data
mbrain_obj <- createSlide(count_mat = mbrain_raw, 
                          slide_info = mbrain_slide_info)
visualizeSlide(slide_obj = mbrain_obj)
visualizeHeatmap(mbrain_obj,rownames(mbrain_raw)[1])

# Decontaminate raw data
decont_obj <- spotclean(mbrain_obj)

# Visualize decontaminated gene
visualizeHeatmap(decont_obj,rownames(mbrain_raw)[1])

# Visualize the estimated per-spot contamination rate
visualizeHeatmap(decont_obj,metadata(decont_obj)$contamination_rate, 
                 logged = FALSE, legend_title = "contamination rate",
                 legend_range = c(0,1))

# (Optionally) Transform to Seurat object for downstream analyses
seurat_obj <- convertToSeurat(decont_obj,image_dir = "/path/to/spatial/folder")
```

## Working with the `SpatialExperiment` class

[`SpatialExperiment`](https://bioconductor.org/packages/release/bioc/html/SpatialExperiment.html) is another widely used S4 class for storing ST data, including the count matrix, cell and gene-level metadata, and tissue image. `spotclean()` can be directly applied to `SpatialExperiment` objects constructed from `SpatialExperiment::read10xVisium()`, which reads in the raw data from 10x Visium Space Ranger output. The visualization functions in our package currently do not support `SpatialExperiment` class, but there are alternative options such as [`ggspavis`](https://bioconductor.org/packages/release/bioc/html/ggspavis.html).

```{r, eval=F}
# Not run
library(SpatialExperiment)
slide_obj <- read10xVisium(samples = "/path/to/spaceranger/output/", 
                         data = "raw") # must specify data = "raw"
decont_obj <- spotclean(slide_obj)
str(assays(decont_obj)$decont)
```

## Running Speed

The computational speed is related to the size and structure of input datasets, mainly driven by the number of tissue spots. SpotClean does not require parallel computation, and thus does not use up too many CPU or memory resources. As a reference, SpotClean running on a medium-size dataset (around 30,000 genes and 2,000 tissue spots) under default gene filtering takes less than 15 minutes.

## Situations you should think twice about before applying SpotClean

- **Too many tissue spots (not enough background spots)** While the observed data is a single matrix with a fixed number of columns (spots), the number of unknown parameters is proportional to the number of tissue spots. In the extreme case where all spots are covered by tissue, we have more unknown parameters than observed data values. In this case the contaminated expressions are confounded with true expressions, and SpotClean estimation becomes unreliable. We recommend that the input data have at least 25% spots not occupied by tissue, so that SpotClean has enough information from background spots to estimate contamination.

- **Lowly-expressed genes** Lowly-expressed genes typically contain relatively less information and relatively more noise than highly-expressed genes. SpotClean by default only keeps highly-expressed or highly-variable genes for decontamination (or both). It can be forced to run on manually-specified lowly-expressed genes. However, even in this case, expression for the lowly-expressed genes is typically not changed very much. Given the high sparsity in most lowly expressed genes, there is not enough information available to confidently reassign UMIs in most cases. However, we do not filter genes by sparsity because there can be interesting genes highly concentrated in a small tissue region. In cases like this, SpotClean is effective at adjusting for spot swapping in these regions.  If the defaults are not appropriate, users can either adjust the expression cutoffs or manually specify genes to decontaminate.

- **Inference based on sequencing depth** SpotClean reassigns bled-out UMIs to their tissue spots of origin which changes the estimated sequencing depth of tissue spots after decontamination, since most estimations of sequencing depth rely on total expressions at every spot. As a result, decontamination can be considered as another type of normalization and might conflict with existing sequencing depth normalization methods.

## Recommended applications

SpotClean leads to improved estimates of expression by correcting for spot swapping. In other words, SpotClean reduces noise by enhancing signal in highly expressed regions and reducing measured signal in unexpressed regions. Consequently, SpotClean will improve the accuracy of marker gene-based inferences including tissue type annotation, cell type decomposition, integration with single-cell RNA-seq data, and associated downstream analyses.  SpotClean also improves the identification of spatially variable and differentially expressed (DE) genes.  We note that in some cases, the p-values associated with known DE genes may increase slightly following SpotClean due to increased variability within the spot groups (i.e. truly expressed regions become more highly expressed; and signal is removed from unexpressed regions).
 
SpotClean will not alter clusters substantially in most datasets given that clusters are largely determined by relatively few highly expressed genes. While clusters may become slightly better defined, in most cases we do not see big differences in the number of clusters and/or relationships among clusters after applying SpotClean.

# Detailed Steps

## Load count matrix and slide information from 10x Space Ranger output

Currently, the most widely-used spatial transcriptomics protocol is 10x Visium. Public 10x datasets can be found [here](https://support.10xgenomics.com/spatial-gene-expression/datasets/). In this vignette, we illustrate the usage of `SpotClean` using the [V1_Adult_Mouse_Brain](https://support.10xgenomics.com/spatial-gene-expression/datasets/1.0.0/V1_Adult_Mouse_Brain) Visium data. 

Two parts of 10x Space Ranger output files are required as input to `SpotClean`: the raw gene-by-spot count matrix, and the slide metadata. In this example, you can download and unzip the [Feature / cell matrix (raw)](https://cf.10xgenomics.com/samples/spatial-exp/1.0.0/V1_Adult_Mouse_Brain/V1_Adult_Mouse_Brain_raw_feature_bc_matrix.tar.gz) and [Spatial imaging data](https://cf.10xgenomics.com/samples/spatial-exp/1.0.0/V1_Adult_Mouse_Brain/V1_Adult_Mouse_Brain_spatial.tar.gz) from [V1_Adult_Mouse_Brain](https://support.10xgenomics.com/spatial-gene-expression/datasets/1.0.0/V1_Adult_Mouse_Brain). You will get a folder `raw_feature_bc_matrix` containing `barcodes.tsv.gz`, `features.tsv.gz`, `matrix.mtx.gz`, and a folder `spatial` containing `aligned_fiducials.jpg`, `detected_tissue_image.jpg`, `tissue_hires_image.png`, `tissue_lowres_image.png`, `tissue_positions_list.csv`, `scalefactors_json.json`. The former folder contains the raw gene-by-spot count matrix, and the latter contains slide metadata like slide images and spot IDs and positions.

`SpotClean` provides functions `read10xRaw()` and `read10xSlide()` to directly read these 10x Space Ranger output files and generate a gene-by-spot count matrix and slide metadata in R. Here for the purpose of demonstration, we have provided the count matrix and slide metadata with our package. The count matrix is a built-in example object `mbrain_raw`, which is a subset of the gene-by-spot count matrix from `read10xRaw()`, containing the top 100 highest expressed genes. The slide metadata, located at `extdata/V1_Adult_Mouse_Brain_spatial`, contains the three basic files (`tissue_positions_list.csv`, `tissue_lowres_image.png`, `scalefactors_json.json`) to run SpotClean and other associated packages in this vignette. 

```{r}
# load count matrix
data(mbrain_raw)
str(mbrain_raw)

# read spatial metadata
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
list.files(spatial_dir)

mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))
str(mbrain_slide_info)
```

## Create the slide object

In the following, we will show the `SpotClean` workflow using the built-in example data. 

We combine count matrix and slide metadata together as one single slide object with class `SummarizedExperiment`:

```{r}
slide_obj <- createSlide(mbrain_raw, mbrain_slide_info)
slide_obj
```

As shown above, the raw count matrix is stored in the `raw` assay slot, and slide and image information are stored in `slide` and `grob` metadata slots. 

## Visualize the slide object

Our package provides multiple visualization functions in the 2-D slide space. Function `visualizeSlide()` shows the input slide imaging file (if given) in R:

```{r,fig.width=5, fig.height=5}
visualizeSlide(slide_obj)
```

Function `visualizeLabel()` shows the spot labels. You can specify the column name of character labels in the `slide` metadata, or manually provide a vector of character labels corresponding to each spot. For example, we can plot their tissue/background labels, which has been pre-stored in the input slide information:

```{r,fig.width=5, fig.height=4}
visualizeLabel(slide_obj,"tissue")
```


Function `visualizeHeatmap()` draws a heatmap of values at every spot in the 2-D slide space. Similar to `visualizeLabel()`, you can specify the column name of numerical values in the `slide` metadata, or manually provide a vector of numerical values corresponding to each spot. For example, we can plot the total UMI counts in every spot,:

```{r,fig.width=5, fig.height=4}
metadata(slide_obj)$slide$total_counts <- Matrix::colSums(mbrain_raw)
visualizeHeatmap(slide_obj,"total_counts")
```

You can also provide a certain gene name appearing in the raw count matrix in input slide object to `visualizeHeatmap()`. For example, the expression of the Mbp gene can be visualized:

```{r,fig.width=5, fig.height=4}
visualizeHeatmap(slide_obj,"Mbp")
```

`visualizeLabel()` and `visualizeHeatmap()` both support manual label/value inputs, subsetting spots to plot, title and legend name modification. `visualizeHeatmap()` also supports different color palettes (rainbow vs. viridis) and log-scaling options. These visualization functions return `ggplot2` objects, which can be further modified by users.

## Decontaminate the data

`spotclean()` is the main function for performing decontamination. It takes the slide object of raw data as input together with some parameters for controlling optimization and convergence, and returns a slide object with decontaminated gene expressions and other model-related parameters and statistics appending to the slide information. Detailed parameter explanations can be found by running `?spotclean`. Here we set `maxit=10` and `candidate_radius=20` to save computation time. In practice, `spotclean()` by default evaluates a series of candidate radii and automatically chooses the best one. The default maximum number of iterations is 30, which can be extended if convergence has not been reached.

```{r}
decont_obj <- spotclean(slide_obj, maxit=10, candidate_radius = 20)
```

Check the structure of output slide object:

```{r}
decont_obj
names(metadata(decont_obj))
```

The metadata now contains more information including parameter estimates from the SpotClean model and measurements of contamination levels. 

We can visualize the Mbp gene expressions after 10 iterations of decontamination:

```{r,fig.width=5, fig.height=4}
visualizeHeatmap(decont_obj,"Mbp")
```

## Estimate contamination levels in observed data

Our model is able to estimate the proportion of contaminated expression at each tissue spot (i.e. expression at a tissue spot that originated from a different spot due to spot swapping):

```{r}
summary(metadata(decont_obj)$contamination_rate)
```

This indicates around 30% of UMIs at a tissue spot in the observed data came from spot swapping contamination, averaging across all tissue spots.

## ARC score

We also provide another subjective estimation of contamination level, called the ambient RNA contamination (ARC) score. It can be calculated using function `arcScore()`, and is also part of the decontamination outputs. Intuitively, the ARC score is a conserved lower bound of the proportion of contamination in observed tissue spots. The ARC score can also be applied in droplet-based single-cell data to estimate ambient RNA contamination when replacing background spots with empty droplets. Details can be found by running `?arcScore`.

```{r}
arcScore(slide_obj)
```

This indicates at least 5% expressions in observed data came from spot swapping contamination.

## Convert to Seurat object for downstream analyses

`convertToSeurat()` can be used to convert our slide object to a Seurat spatial object. Note that Seurat requires input of the spatial folder. 

```{r}
seurat_obj <- convertToSeurat(decont_obj,image_dir = spatial_dir)
```

# Session Information

```{r}
sessionInfo()
```

# Citation

`SpotClean` can be cited at:

Ni, Z., Prasad, A., Chen, S. et al. SpotClean adjusts for spot swapping in spatial transcriptomics data. *Nat Commun* 13, 2971 (2022). https://doi.org/10.1038/s41467-022-30587-y

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_contamination_score.R
\name{arcScore}
\alias{arcScore}
\alias{arcScore.default}
\alias{arcScore.SummarizedExperiment}
\title{Calculate the ambient RNA contamination (ARC) score}
\usage{
arcScore(object, ...)

\method{arcScore}{default}(object, background_bcs, ...)

\method{arcScore}{SummarizedExperiment}(object, ...)
}
\arguments{
\item{object}{A gene-by-barcode count matrix or a slide object created or
inherited from \code{createSlide()}.
Ideally this should give the raw count matrix with all genes and barcodes
without any filtering.}

\item{...}{Arguments passed to other methods}

\item{background_bcs}{(vector of chr) Background barcodes in
\code{count_mat}. For spatial data, these are background spots not
covered by tissue. For single-cell data, these are empty droplets not
containing cells.}
}
\value{
(num) The ARC score of given data.
}
\description{
The ARC score is intended for subjectively measuring the level
of contamination caused by ambient RNAs for a given spatial transcriptomics
or droplet-based single-cell RNA-seq data. Intuitively, this score is a lower
bound of the average proportion of contaminated expressions in tissue
spots or cell droplets in the observed contaminated data.

ARC score is calculated as follows: (1) estimate the average amount of
contaminated UMI counts received per spot (droplet) using the total UMI
counts in background spots (empty droplets) divided by total number of
spots (droplets). This is an underestimation as it does not account for
contamination inside tissue spots or cell droplets. (2) estimate the average
amount of observed UMI counts per tissue spot (cell droplet) using the
total UMI counts in tissue spots (cell droplets) divided by number of
tissue spots (cell droplets). (3) Divide the value in (1) by the value in
(2) to get the ARC score.

This lower bound is very conservative as it only
accounts for observable contamination in background spots or empty droplets,
neglecting contamination in tissue spots or cell droplets. However, it
is totally subjective, not relying on any complicated assumptions and
modelling. ARC score serves as a relative contamination measurement for
comparison among different datasets and protocols.
}
\examples{

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))

background_bcs <- dplyr::filter(mbrain_slide_info$slide, tissue==0)$barcode
arcScore(mbrain_raw, background_bcs)

mbrain_obj <- createSlide(mbrain_raw, mbrain_slide_info)
arcScore(mbrain_obj)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_to_seurat.R
\name{convertToSeurat}
\alias{convertToSeurat}
\title{Convert slide object to Seurat object}
\usage{
convertToSeurat(slide_obj, image_dir, slice = "slice1", filter_matrix = TRUE)
}
\arguments{
\item{slide_obj}{A slide object created or inherited from
\code{createSlide()}.}

\item{image_dir}{(chr) Path to directory with 10X Genomics visium image data;
should include files \code{tissue_lowres_image.png},
\code{scalefactors_json.json} and \code{tissue_positions_list.csv}.}

\item{slice}{(chr) Name for the stored image of the tissue slice.
Default: "slice1"}

\item{filter_matrix}{(logical) If \code{TRUE}, only keep spots that have been
determined to be over tissue. If \code{slide_obj} only contains tissue spots,
\code{filter_matrix} has to be set \code{TRUE}. If \code{slide_obj}
contains both tissue and background spots, setting \code{filter_matrix=TRUE}
will subset the expression matrix to tissue spots only. Default: \code{TRUE}}
}
\value{
A Seurat object with spatial information.
}
\description{
This function converts our slide object of class
\code{SummarizedExperiment} to Seurat object of class \code{Seurat} so that
users can directly proceed with Seurat spatial analyses pipelines.
Built based on Seurat's \code{Load10X_Spatial()}.
}
\examples{

# load count matrix and slide metadata
data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))

# Create slide object
mbrain_obj <- createSlide(mbrain_raw,
                          mbrain_slide_info)

# Convert to Seurat object
seurat_obj <- convertToSeurat(mbrain_obj, spatial_dir, "raw")
str(seurat_obj)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_label.R
\name{visualizeLabel}
\alias{visualizeLabel}
\alias{visualizeLabel.default}
\alias{visualizeLabel.SummarizedExperiment}
\title{Visualize spot labels on the 2D slide}
\usage{
visualizeLabel(object, ...)

\method{visualizeLabel}{default}(
  object,
  label = "tissue",
  subset_barcodes = NULL,
  title = "",
  legend_title = NULL,
  ...
)

\method{visualizeLabel}{SummarizedExperiment}(
  object,
  label = "tissue",
  subset_barcodes = NULL,
  title = "",
  legend_title = NULL,
  ...
)
}
\arguments{
\item{object}{A slide object created or
inherited from \code{createSlide()}, or a \code{data.frame} of slide
information with columns: barcodes, tissue, imagerow, imagecol, etc.}

\item{...}{Arguments passed to other methods}

\item{label}{(categorical vector or chr) Either a vector of labels for all
spots, or the column name in the \code{slide} slot of the slide
object's metadata. In the former case, the order of values in the vector
should match the spot barcodes in the slide object. In the latter case,
The column should be categorical with reasonably number of unique labels.
Default: \code{"tissue"}}

\item{subset_barcodes}{(vector of chr) A subset of spot barcodes to plot.
By default it plots all spots in the slide object. This can be useful
when only plotting tissue spots or specific tissue types or regions.
Default: \code{NULL}}

\item{title}{(chr) Title of the plot. Default: \code{""}}

\item{legend_title}{(chr) Title of the legend. Under default,
use \code{label} as legend title. Default: \code{NULL}}
}
\value{
A \code{ggplot2} object.
}
\description{
Generate and visualize spot labels on the 2D slide
as a ggplot2 object. Spot labels can be given via parameter \code{label} as
external input or one column in the \code{slide} slot of the slide
object's metadata via \code{label_col}. Exactly one of \code{label}
and \code{label_col} must be specified.
}
\examples{

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))
mbrain_obj <- createSlide(mbrain_raw,
                          mbrain_slide_info)
gp <- visualizeLabel(mbrain_obj, label="tissue",
                     title="mbrain", legend_title="tissue or background")
plot(gp)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_slide.R
\name{createSlide}
\alias{createSlide}
\title{Create a new slide object}
\usage{
createSlide(count_mat, slide_info, gene_cutoff = 0.1, verbose = TRUE)
}
\arguments{
\item{count_mat}{(matrix of num) The raw gene-by-barcode count matrix.
Can be either standard matrix format or sparse matrix format.}

\item{slide_info}{(list or data.frame) A list of slide information from
\code{read10xSlide()}, or a data frame only containing spot information
like barcode, tissue, imagerow, imagecol, etc.}

\item{gene_cutoff}{(num) Filter out genes with average expressions
among tissue spots below or equal to this cutoff.
Default: 0.1}

\item{verbose}{(logical) Whether print progress information.
Default: \code{TRUE}}
}
\value{
A \code{SummarizedExperiment} object containing
gene expression and spot metadata.
}
\description{
This function takes input of the count matrix
(from \code{read10xRaw} or \code{read10xRawH5}) and slide information
(from \code{read10xSlide} or manually specified data frame) and outputs
a \code{SummarizedExperiment} object
as our slide object for downstream decontamination and visualization.
}
\examples{

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))

mbrain_obj <- createSlide(mbrain_raw,
                          mbrain_slide_info)
mbrain_obj
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decontamination.R
\name{spotclean}
\alias{spotclean}
\alias{spotclean.SummarizedExperiment}
\alias{spotclean.SpatialExperiment}
\title{Decontaminate spot swapping effect in spatial transcriptomics data}
\usage{
spotclean(slide_obj, ...)

\method{spotclean}{SummarizedExperiment}(
  slide_obj,
  gene_keep = NULL,
  maxit = 30,
  tol = 1,
  candidate_radius = 5 * seq_len(6),
  kernel = "gaussian",
  verbose = TRUE,
  ...
)

\method{spotclean}{SpatialExperiment}(
  slide_obj,
  gene_keep = NULL,
  gene_cutoff = 0.1,
  maxit = 30,
  tol = 1,
  candidate_radius = 5 * seq_len(6),
  kernel = "gaussian",
  verbose = TRUE,
  ...
)
}
\arguments{
\item{slide_obj}{A slide object created or inherited from
\code{createSlide()}, or a \code{SpatialExperiment} object created from
\code{SpatialExperiment::read10xVisium()}.}

\item{...}{Arguments passed to other methods}

\item{gene_keep}{(vector of chr) Gene names to keep for decontamination.
We recommend not decontaminating lowly expressed and lowly variable genes
in order to save computation time. Even if user include them, their
decontaminated expressions will not change too much from raw expressions.
When setting to \code{NULL}, \code{keepHighGene()} will be
automatically called to filter out lowly expressed and lowly variable genes
before decontamination.
Default: \code{NULL}.}

\item{maxit}{(int) Maximum iteration for EM parameter updates. Default: 30.}

\item{tol}{(num) Tolerance to define convergence in EM parameter updates.
When the element-wise maximum difference between current and updated
parameter matrix is less than \code{tol}, parameters are considered
converged. Default: 1}

\item{candidate_radius}{(vector of num) Candidate contamination radius.
A series of radius to try when estimating contamination parameters.
Default: \code{c(5, 10, 15, 20, 25, 30)}}

\item{kernel}{(chr): name of kernel to use to model local contamination.
Supports "gaussian", "linear", "laplace", "cauchy". Default: "gaussian".}

\item{verbose}{(logical) Whether print progress information.
Default: \code{TRUE}}

\item{gene_cutoff}{(num) Filter out genes with average expressions
among tissue spots below or equal to this cutoff.
Only applies to \code{SpatialExperiment} object.
Default: 0.1.}
}
\value{
For slide object created from \code{createSlide()}, returns a
slide object where the decontaminated expression matrix is in the
"decont" assay slot and the contamination statistics are in
metadata slots. Contamination statistics include ambient RNA contamination
(ARC) score, bleeding rate, distal rate, contamination radius,
contamination kernel weight matrix, log-likelihood value in each iteration,
estimated proportion of contamination in each tissue spot in observed data.
Since decontaminated and raw data have different number of columns, they can
not be stored in a single object.

For \code{SpatialExperiment} object created from
\code{SpatialExperiment::read10xVisium()}, returns a
\code{SpatialExperiment} object where the decontaminated expression matrix
is in the "decont" assay slot and the contamination statistics are in
metadata slots. Raw expression matrix is also stored in the "counts" assay
slot. Genes are filtered based on \code{gene_cutoff}.
}
\description{
This is the main function implementing the SpotClean method
for decontaminating spot swapping effect in spatial transcriptomics data.
}
\details{
Briefly, the contamination level for the slide is estimated based on
the total counts of all spots. UMI counts travelling around the slide are
assumed to follow Poisson distributions and modeled by a mixture of
Gaussian (proximal) and uniform (distal) kernels. The underlying
uncontaminated gene expressions are estimated by EM algorithm to
maximize the data likelihood. Detailed derivation can be found in our
manuscript.
}
\examples{

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))
mbrain_obj <- createSlide(mbrain_raw,
                          mbrain_slide_info)

mbrain_decont_obj <- spotclean(mbrain_obj, tol=10, candidate_radius=20)
mbrain_decont_obj
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read10x.R
\name{read10xRaw}
\alias{read10xRaw}
\alias{read10xRawH5}
\alias{read10xSlide}
\title{Read 10x Space Ranger output data}
\usage{
read10xRaw(count_dir = NULL, row_name = "symbol", meta = FALSE)

read10xRawH5(h5_file, row_name = "symbol", meta = FALSE)

read10xSlide(tissue_csv_file, tissue_img_file = NULL, scale_factor_file = NULL)
}
\arguments{
\item{count_dir}{(chr) The directory of 10x output matrix data.
The directory should include
three files: barcodes.tsv.gz, features.tsv.gz, matrix.mtx.gz.}

\item{row_name}{(chr) Specify either using gene symbols
(\code{row_name = "symbol"}) or gene Ensembl IDs (\code{row_name = "id"})
as row names of the count matrix.
Default: \code{row_name = "symbol"}}

\item{meta}{(logical) If \code{TRUE}, \code{read10xRaw} or
\code{read10xRawH5} returns a list containing both the
count matrix and metadata of genes (features). Metadata includes feature
names, IDs and other additional information depending on Space Ranger
output. If \code{FALSE}, only returns the count matrix.
Default: \code{FALSE}}

\item{h5_file}{(chr) The path of 10x output matrix HDF5 file
(ended with .h5).}

\item{tissue_csv_file}{(chr) The path of 10x output CSV file of
spot positions, usually named \code{tissue_positions_list.csv} for 
Space Ranger V1 and \code{tissue_positions.csv} for Space Ranger V2.}

\item{tissue_img_file}{(chr) The path of the 10x output low resolution
tissue image in PNG format,
usually named \code{tissue_lowres_image.png}. If \code{NULL},
the returned slide data does not contain image
information. Please do provide this file if you could find it.
Default: \code{NULL}}

\item{scale_factor_file}{(chr) The path of the 10x output scale factor
file in json format, usually named \code{scalefactors_json.json}.
If \code{NULL}, spot positions in image will
not be corrected by the scale factor. Please do provide this file
if you could find it. Default: \code{NULL}}
}
\value{
If \code{meta = TRUE}, \code{read10xRaw()} or \code{read10xRawH5()}
returns a list of two elements: a
"dgCMatrix" sparse matrix containing expression counts and a data
frame containing metadata of genes (features). For the count matrix,
each row is a gene (feature) and each column is a spot barcode. If
\code{meta = FALSE}, only returns the count matrix.

\code{read10xSlide()} returns a list of two objects. The first object, slide,
is a data.frame where each row corresponds to a spot
and each column corresponds to slide information such as row and column
positions on the slide. The second object, grob, is a Grid Graphical Object
of the tissue image when specifying \code{tissue_img_file}.
}
\description{
\code{read10xRaw()} is a one-line handy function for reading
the raw expression data from 10x Space Ranger outputs and producing a count
matrix as an R object.

\code{read10xRawH5()} is for reading 10x Space Ranger
output HDF5 file (ended with .h5).

\code{read10xSlide()} is for reading slide
information (e.g. spot positions) and the tissue image from 10x
Space Ranger outputs. This function is developed based on 10x's secondary
analysis pipeline
https://support.10xgenomics.com/spatial-gene-expression/software/pipelines/latest/rkit.
}
\examples{

# simulate 10x output files of count matrix
data(mbrain_raw)
data_dir <- file.path(tempdir(),"sim_example")
dir.create(data_dir)
matrix_dir <- file.path(data_dir,"matrix.mtx")
barcode_dir <- gzfile(file.path(data_dir, "barcodes.tsv.gz"), open="wb")
gene_dir <- gzfile(file.path(data_dir, "features.tsv.gz"), open="wb")

# For simplicity, use gene names to generate gene IDs to fit the format.
gene_name <- rownames(mbrain_raw)
gene_id <- paste0("ENSG_fake_",gene_name)
barcode_id <- colnames(mbrain_raw)

Matrix::writeMM(mbrain_raw,file = matrix_dir)
write(barcode_id,file = barcode_dir)
write.table(cbind(gene_id,gene_name,"type"),file = gene_dir,
    sep = "\t", quote = FALSE, col.names = FALSE, row.names = FALSE)
R.utils::gzip(matrix_dir)
close(barcode_dir)
close(gene_dir)


# read expression count matrix
list.files(data_dir)
mbrain_raw_new <- read10xRaw(data_dir)
str(mbrain_raw_new)
identical(mbrain_raw, mbrain_raw_new)

# read slide metadata
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
list.files(spatial_dir)
mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))
str(mbrain_slide_info)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_heatmap.R
\name{visualizeHeatmap}
\alias{visualizeHeatmap}
\alias{visualizeHeatmap.default}
\alias{visualizeHeatmap.SummarizedExperiment}
\title{Visualize spot values on the 2D slide}
\usage{
visualizeHeatmap(object, ...)

\method{visualizeHeatmap}{default}(
  object,
  value,
  exp_matrix = NULL,
  subset_barcodes = NULL,
  logged = TRUE,
  viridis = TRUE,
  legend_range = NULL,
  title = "",
  legend_title = NULL,
  ...
)

\method{visualizeHeatmap}{SummarizedExperiment}(
  object,
  value,
  subset_barcodes = NULL,
  logged = TRUE,
  viridis = TRUE,
  legend_range = NULL,
  title = "",
  legend_title = NULL,
  ...
)
}
\arguments{
\item{object}{A slide object created or
inherited from \code{createSlide()}, or a \code{data.frame} of slide
information with columns: barcodes, tissue, imagerow, imagecol, etc.}

\item{...}{Arguments passed to other methods}

\item{value}{(numeric vector or chr) Either a vector of numeric values for
all spots, or the name of one gene in \code{exp_matrix} or in the expression
matrix in the slide object.
In the former case, the order of values in the
vector should match the spot barcodes in the slide object.}

\item{exp_matrix}{(matrix of num) When \code{object} is a data frame and
\code{value} is a single character
string, will search the matching gene in \code{exp_matrix} and plot the gene
expression. Default: \code{NULL}}

\item{subset_barcodes}{(vector of chr) A subset of spot barcodes to plot.
By default it plots all spots in the slide object. This can be useful
when only plotting tissue spots or specific tissue types or regions.
Default: \code{NULL}}

\item{logged}{(logical) Specify if the color scale is log1p transformed.
Default: \code{TRUE}}

\item{viridis}{(logical) If true, color scale uses viridis.
Otherwise, use rainbow. Default: \code{TRUE}}

\item{legend_range}{(length 2 vector of num) Custom legend
range of the value. By default uses the range of the plotted values.
Default: \code{NULL}}

\item{title}{(chr) Title of the plot. Default: \code{""}}

\item{legend_title}{(chr) Title of the legend. Under default,
use \code{value} as legend title. Default: \code{NULL}}
}
\value{
A \code{ggplot2} object.
}
\description{
Generate and visualize spot labels on the 2D slide
as a ggplot2 object. Spot labels can be given via parameter \code{label} as
external input or one column in the \code{slide} slot of the slide
object's metadata via \code{label_col}. Exactly one of \code{label}
and \code{label_col} must be specified.
}
\examples{

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))
mbrain_obj <- createSlide(mbrain_raw,
                          mbrain_slide_info)

gp <- visualizeHeatmap(mbrain_obj, "Bc1",
                     title="mbrain", legend_title="Bc1 expression")
plot(gp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mbrain_raw.R
\docType{data}
\name{mbrain_raw}
\alias{mbrain_raw}
\title{Example 10x Visium spatial data: raw count matrix}
\format{
An object of class \code{"dgCMatrix"}.
}
\source{
\href{https://support.10xgenomics.com/spatial-gene-expression/datasets/1.0.0/V1_Adult_Mouse_Brain}{V1_Adult_Mouse_Brain}
}
\usage{
data(mbrain_raw)
}
\description{
This dataset contains one sparse count matrix
\code{mbrain_raw} of gene expressions. The original dataset can be found at
https://support.10xgenomics.com/spatial-gene-expression/datasets/1.0.0/V1_Adult_Mouse_Brain.
For simplicity, we only keep the top 100 highest expressed genes in this
example data.

If users read raw data using \code{read10xRaw} (or \code{read10xRawH5}),
they should get exactly the same format as the object in this dataset.
}
\examples{
data(mbrain_raw)
str(mbrain_raw)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/keep_high_gene.R
\name{keepHighGene}
\alias{keepHighGene}
\title{Filter and return highly expressed or highly variable genes}
\usage{
keepHighGene(
  count_mat,
  top_high = 5000,
  mean_cutoff = 1,
  return_matrix = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{count_mat}{(matrix of num) Input count matrix to be filtered.
Can be either standard matrix format or sparse matrix format.}

\item{top_high}{(int) Only look for highly expressed and variable genes
within this number of top expressed genes. Default: 5000.}

\item{mean_cutoff}{(num) Genes with average expressions among all
spots exceeding this cutoff are kept as highly expressed genes.}

\item{return_matrix}{(logical) Whether return filtered matrix instead
of gene names. Default: \code{FALSE}}

\item{verbose}{(logical) Whether print progress information.
Default: \code{TRUE}}
}
\value{
A vector of gene names or a filtered expression count
matrix with the same class as \code{count_mat}.
}
\description{
This function is for filtering genes in the expression count
matrix based on their average expression and variability. Usually genes
with low expressions and variability are less interesting and do not
contribute too much to downstream analyses, but rather bring technical noise.
It is always recommended to pre-filter gene expression matrix before
any analyses.

We apply the
function \code{FindVariableFeatures} with \code{selection.method = "mvp"}
from package \code{Seurat} on log transformed expression matrix to
detect high variable genes. This method does not require a pre-specified
number of high variable genes.
}
\examples{

data(mbrain_raw)
dim(mbrain_raw)

mbrain_raw_f <- keepHighGene(mbrain_raw, mean_cutoff=100,
                             return_matrix=TRUE)
dim(mbrain_raw_f)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_slide.R
\name{visualizeSlide}
\alias{visualizeSlide}
\title{Visualize the Visium slide image}
\usage{
visualizeSlide(slide_obj, title = "")
}
\arguments{
\item{slide_obj}{A slide object created or inherited from
\code{createSlide()}.}

\item{title}{(chr) Title of the plot. Default: \code{""}}
}
\value{
A \code{ggplot2} object.
}
\description{
Generate and visualize the tissue image as a ggplot2 object.
Users can manually add and modify layers (e.g. title, axis)
following ggplot2's syntax.
}
\examples{

data(mbrain_raw)
spatial_dir <- system.file(file.path("extdata",
                                     "V1_Adult_Mouse_Brain_spatial"),
                           package = "SpotClean")
mbrain_slide_info <- read10xSlide(tissue_csv_file=file.path(spatial_dir,
                                       "tissue_positions_list.csv"),
             tissue_img_file = file.path(spatial_dir,
                                       "tissue_lowres_image.png"),
             scale_factor_file = file.path(spatial_dir,
                                       "scalefactors_json.json"))
mbrain_obj <- createSlide(mbrain_raw,
                          mbrain_slide_info)
gp <- visualizeSlide(mbrain_obj)
plot(gp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SpotClean-package.R
\docType{package}
\name{SpotClean-package}
\alias{SpotClean}
\alias{SpotClean-package}
\title{SpotClean: a computational method to adjust for spot swapping in
spatial transcriptomics data}
\description{
SpotClean is a computational method to adjust for spot swapping in
spatial transcriptomics data. Recent spatial transcriptomics experiments
utilize slides containing thousands of spots with spot-specific barcodes
that bind mRNA. Ideally, unique molecular identifiers at a spot measure
spot-specific expression, but this is often not the case due to bleed from
nearby spots, an artifact we refer to as spot swapping. SpotClean is able
to estimate the contamination rate in observed data and decontaminate the
spot swapping effect, thus increase the sensitivity and precision of
downstream analyses.
}
\details{
To learn more about \code{SpotClean}, see the vignette
using \code{browseVignettes(package = "SpotClean")}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/zijianni/SpotClean}
  \item Report bugs at \url{https://github.com/zijianni/SpotClean/issues}
}

}
\author{
\strong{Maintainer}: Zijian Ni \email{zni25@wisc.edu} (\href{https://orcid.org/0000-0003-1181-8337}{ORCID})

Other contributors:
\itemize{
  \item Christina Kendziorski [contributor]
}

}
